/**********************************************************************/
/*                         W H U F F M A N . C P P                 	 */
/*--------------------------------------------------------------------*/
/*    Fonction       :   Codage  longueur variable	                */
/*--------------------------------------------------------------------*/
/*    Auteur         :   MC                			     					 */
/*    Dvelopp le   :   29/04/97                                     */
/*    Dernire modif.:   23/05/97                                     */
/**********************************************************************/

#include <stdlib.h>
#include "whuffman.h"

/*----------------------------------------------*/
/*																*/
/*		Constantes du programme        				*/
/*															  	*/
/*----------------------------------------------*/

/* XHUFF contains all the information that needs be transmitted */
/* EHUFF and DHUFF are derivable from XHUFF */

typedef struct {
long bits[36];          /* Bit-length frequency (indexed on length  */
long huffval[257];      /* Huffman value index */
}XHUFF ;

/* Encoder tables */

typedef struct {
long ehufco[257];      /* Encoder huffman code indexed on code word */
char ehufsi[257];      /* Encoder huffman code-size indexed on code word */
}EHUFF;

/* Decoder tables */

typedef struct {
int ml;               /* Maximum length */
long maxcode[36];      /* Max code for a given bit length -1 if no codes */
long mincode[36];      /* Min code for a given bit length */
long valptr[36];       /* First index (of min-code) for a given bit-length */
}DHUFF ;

/*----------------------------------------------*/
/*																*/
/*		Variables du programme     			     */
/*																*/
/*----------------------------------------------*/

int LuminanceDCBits[] = {
0x00, 0x01, 0x05, 0x01, 0x01, 0x01, 0x01, 0x01,
0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};

int LuminanceDCValues[] = {
0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b};

int LuminanceACBits[] = {
0x00, 0x02, 0x01, 0x03, 0x03, 0x02, 0x04, 0x03,
0x05, 0x05, 0x04, 0x04, 0x00, 0x00, 0x01, 0x7d};

int LuminanceACValues[] = {
0x01, 0x02, 0x03, 0x00, 0x04, 0x11, 0x05, 0x12,
0x21, 0x31, 0x41, 0x06, 0x13, 0x51, 0x61, 0x07,
0x22, 0x71, 0x14, 0x32, 0x81, 0x91, 0xa1, 0x08,
0x23, 0x42, 0xb1, 0xc1, 0x15, 0x52, 0xd1, 0xf0,
0x24, 0x33, 0x62, 0x72, 0x82, 0x09, 0x0a, 0x16,
0x17, 0x18, 0x19, 0x1a, 0x25, 0x26, 0x27, 0x28,
0x29, 0x2a, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39,
0x3a, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49,
0x4a, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59,
0x5a, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69,
0x6a, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79,
0x7a, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89,
0x8a, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98,
0x99, 0x9a, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7,
0xa8, 0xa9, 0xaa, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6,
0xb7, 0xb8, 0xb9, 0xba, 0xc2, 0xc3, 0xc4, 0xc5,
0xc6, 0xc7, 0xc8, 0xc9, 0xca, 0xd2, 0xd3, 0xd4,
0xd5, 0xd6, 0xd7, 0xd8, 0xd9, 0xda, 0xe1, 0xe2,
0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea,
0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8,
0xf9, 0xfa};

long lmask[] = {        /* This is 2^{i+1}-1 */
0x00000001L,0x00000003L,0x00000007L,0x0000000fL,
0x0000001fL,0x0000003fL,0x0000007fL,0x000000ffL,
0x000001ffL,0x000003ffL,0x000007ffL,0x00000fffL,
0x00001fffL,0x00003fffL,0x00007fffL,0x0000ffffL,
0x0001ffffL,0x0003ffffL,0x0007ffffL,0x000fffffL,
0x001fffffL,0x003fffffL,0x007fffffL,0x00ffffffL,
0x01ffffffL,0x03ffffffL,0x07ffffffL,0x0fffffffL,
0x1fffffffL,0x3fffffffL,0x7fffffffL,0xffffffffL};

/* Masks */
long bit_set_mask[] = { /* This is 2^i at ith position */
0x00000001L,0x00000002L,0x00000004L,0x00000008L,
0x00000010L,0x00000020L,0x00000040L,0x00000080L,
0x00000100L,0x00000200L,0x00000400L,0x00000800L,
0x00001000L,0x00002000L,0x00004000L,0x00008000L,
0x00010000L,0x00020000L,0x00040000L,0x00080000L,
0x00100000L,0x00200000L,0x00400000L,0x00800000L,
0x01000000L,0x02000000L,0x04000000L,0x08000000L,
0x10000000L,0x20000000L,0x40000000L,0x80000000L};

char huffsize[257];
long huffcode[257];
int lastp;

XHUFF *Xhuff=NULL;           // pointeurs sur les tables
DHUFF *Dhuff=NULL;           // dcodeur
EHUFF *Ehuff=NULL;           // codeur

XHUFF *XhuffDC=NULL;           // pointeurs sur les tables
DHUFF *DhuffDC=NULL;           // dcodeur
EHUFF *EhuffDC=NULL;           // codeur

XHUFF *XhuffAC=NULL;           // pointeurs sur les tables
DHUFF *DhuffAC=NULL;           // dcodeur
EHUFF *EhuffAC=NULL;           // codeur

long current_write_byte;
long current_read_byte;
char read_position;
char write_position;

unsigned int i_ecr;             // indices
unsigned int i_lec;
unsigned char far *bufcomp;      // buffer de codes

/*----------------------------------------------*/
/*																*/
/*  	     Fonctions prototypes						*/
/*																*/
/*----------------------------------------------*/

void SpecifiedHuffman(int *bts,int *hvls);
void MakeDecoderHuffman(void);
void SizeTable(void);
void CodeTable(void);
void OrderCodes(void);
void DecoderTables(void);
int AllocTables(void);
int IniTables(void);
int megetb(void);
unsigned char pgetc(void);
void pputc(unsigned char c);

/*-------------------------------------------------------------------------*/
/*                                                                         */
/*                         Procdures du programme                         */
/*       								   													*/
/*-------------------------------------------------------------------------*/

void Inicodec(void)
{
	current_write_byte=0;
	current_read_byte=0;
	read_position=-1;
	write_position=7;

	i_ecr=0;
	i_lec=0;
}

void LibereTables(void)
{
	free(DhuffDC);
	free(EhuffDC);
	free(XhuffDC);

	free(DhuffAC);
	free(EhuffAC);
	free(XhuffAC);
}

int IniTables(void)
{
int i=AllocTables();
	if(i) return(i);

	UseDCHuffman();
	SpecifiedHuffman(LuminanceDCBits,LuminanceDCValues);
	MakeDecoderHuffman();


	UseACHuffman();
	SpecifiedHuffman(LuminanceACBits,LuminanceACValues);
	MakeDecoderHuffman();

	return 0;
}

void UseDCHuffman(void)
{
	Xhuff=XhuffDC;
	Ehuff=EhuffDC;
	Dhuff=DhuffDC;
}

void UseACHuffman(void)
{
	Xhuff=XhuffAC;
	Ehuff=EhuffAC;
	Dhuff=DhuffAC;
}

int AllocTables(void)
{
	XhuffDC =(XHUFF *) malloc(sizeof(XHUFF));if(!XhuffDC) return 1;
	EhuffDC =(EHUFF *) malloc(sizeof(EHUFF));if(!EhuffDC) return 1;
	DhuffDC =(DHUFF *) malloc(sizeof(DHUFF));if(!DhuffDC) return 1;
	XhuffAC =(XHUFF *) malloc(sizeof(XHUFF));if(!XhuffAC) return 1;
	EhuffAC =(EHUFF *) malloc(sizeof(EHUFF));if(!EhuffAC) return 1;
	DhuffAC =(DHUFF *) malloc(sizeof(DHUFF));if(!DhuffAC) return 1;
	return 0;
}

void SpecifiedHuffman(int *bts,int *hvls)
{
  int i;
  int accum;

  for(accum=0,i=0;i<16;i++)
	 {
		accum+= bts[i];
		Xhuff->bits[i+1] = bts[i];  /* Shift offset for internal specs.*/
	 }
  for(i=0;i<accum;i++)
	 {
		Xhuff->huffval[i] = hvls[i];
	 }
  SizeTable();         //From Xhuff to Ehuff
  CodeTable();
  OrderCodes();
}

/*BFUNC

MakeDecoderHuffman() creates the decoder tables from the Xhuff structure.

EFUNC*/

void MakeDecoderHuffman()
{
  SizeTable();
  CodeTable();
  DecoderTables();
}

/*BFUNC

SizeTable() is used to associate a size with the code in increasing
length. For example, it would be 44556677... in huffsize[].  Lastp is
the number of codes used.

EFUNC*/

void SizeTable()
{
int i,j,p;

  for(p=0,i=1;i<17;i++)
    {
		for(j=1;j<=Xhuff->bits[i];j++)
		{
		huffsize[p++] = i;
		}
    }
  huffsize[p] = 0;
  lastp = p;
}


/*BFUNC

CodeTable() is used to generate the codes once the hufsizes are known.

EFUNC*/

void CodeTable()
{
int p,size;
long code;

  p=0;
  code=0;
  size = huffsize[0];
  while(1)
	 {
		do
	{
	  huffcode[p++] = code++;
	}
		while((huffsize[p]==size)&&(p<257)); /* Overflow Detection */
		if (!huffsize[p]) /* All finished. */
	{
	  break;
	}
		do                /* Shift next code to expand prefix. */
	{
	  code <<= 1;
	  size++;
	}
		while(huffsize[p] != size);
	 }
}

/*BFUNC

OrderCodes() reorders from the monotonically increasing Huffman-code
words into an array which is indexed on the actual value represented
by the codes. This converts the Xhuff structure into an Ehuff
structure.

EFUNC*/

void OrderCodes()
{
  int index,p;

  for(p=0;p<lastp;p++)
	 {
		index = (int)Xhuff->huffval[p];
		Ehuff->ehufco[index] = huffcode[p];
		Ehuff->ehufsi[index] = huffsize[p];
	 }
}

/*BFUNC

DecoderTables() takes the Xhuff and converts it to a form suitable for
the JPEG suggested decoder. This is not the fastest method but it is
the reference method.

EFUNC*/

void DecoderTables()
{
  int l,p;

  for(Dhuff->ml=1,p=0,l=1;l<=16;l++)
	 {
		if (Xhuff->bits[l]==0)
	{
	  Dhuff->maxcode[l] = -1; /* Watch out JPEG is wrong here */
	}                         /* We use -1 to indicate skipping. */
		else
	{
	  Dhuff->valptr[l]=p;
	  Dhuff->mincode[l]=huffcode[p];
	  p+=(int)Xhuff->bits[l]-1;
	  Dhuff->maxcode[l]=huffcode[p];
	  Dhuff->ml = l;
	  p++;
	}
	 }
  Dhuff->maxcode[Dhuff->ml]++;
}

/*BFUNC

EncodeHuffman() places the Huffman code for the value onto the stream.

EFUNC*/

char EncodeHuffman(int value)
{
char size;
long code;

	size=Ehuff->ehufsi[value];
	//if(size)
	// {
		code=Ehuff->ehufco[value];
		meputv(size,code);
  //	 }
	//else
  //	 {
	 // erreur
  //	 }

	return(size);  // Retourne le nombre de bits du code
}


/*BFUNC

meputv() puts n bits from b onto the writer stream.

EFUNC*/

void meputv(char n,long data)         // max 16
{
char p;
long b;

  b=data;

  n--;
  b &= lmask[n];
  p = n - write_position;
  if (!p)                           /* Can do parallel save immediately */
	 {
		current_write_byte |= b;
		pputc((unsigned char)current_write_byte);
		current_write_byte = 0;
		write_position = 7;
		return;
	 }
  else if (p < 0)                   /* if can fit, we have to shift byte */
	 {
		p = -p;
		current_write_byte |= (b << p);
		write_position = p-1;
		return;
	 }
  current_write_byte |= (b >> p);  /* cannot fit. we must do putc's */
  pputc((unsigned char)current_write_byte);       /* Save off  remainder */
  while(p > 7)                     /* Save off bytes while remaining > 7 */
	 {
		p -= 8;
		current_write_byte = (b >> p) & lmask[7];
		pputc((unsigned char)current_write_byte);
	 }
  if (!p)                          /* If zero then reset position */
	 {
		write_position = 7;
		current_write_byte = 0;
	 }
  else                             /* Otherwise reset write byte buffer */
	 {
		write_position = 8-p;
		current_write_byte = (b << write_position) & lmask[7];
		write_position--;
	 }
}

/*BFUNC

DecodeHuffman() returns the value decoded from the Huffman stream.
The Dhuff must be loaded before this function be called.

EFUNC*/

long DecodeHuffman()
{
  long code;
  //long c;
  int l,p;

//  code = (long)megetb();           // appel

  if(read_position < 0)
	 {
		current_read_byte = pgetc();
		read_position=7;
	 }
  if(current_read_byte&bit_set_mask[read_position--])
	 {
		code=1;
	 }
  else code=0;

  for(l=1;code>Dhuff->maxcode[l];l++)
	 {
	 //	code=(code<<1)+(long)megetb();  // appel
  if(read_position < 0)
	 {
		current_read_byte = pgetc();
		read_position=7;
	 }
  if(current_read_byte&bit_set_mask[read_position--])
	 {
		code=(code<<1)+1;            // 1
	 }
  else
		code=(code<<1);             // 0
	 }

//  if(code<(long)Dhuff->maxcode[Dhuff->ml])
//	 {
		p = (int)((long)Dhuff->valptr[l] + code - (long)Dhuff->mincode[l]);
		return((long)Xhuff->huffval[p]);
//	 }
//  else
//	 {
	  // erreur
//	 }
}

/*BFUNC

megetb() gets a bit from the read stream.

EFUNC*/

int megetb()
{
  if (read_position < 0)
	 {
		current_read_byte = pgetc();
		read_position=7;
	 }
  if (current_read_byte&bit_set_mask[read_position--])
	 {
		return(1);
	 }
  return(0);
}

/*BFUNC

megetv() gets n bits from the read stream and returns it.

EFUNC*/

long megetv(char n)
{
long rv;
char p;

  n--;
  p = n-read_position;
  while(p > 0)
	 {
		if (read_position>23)  /* If byte buffer contains almost entire word */
		{
		rv = (current_read_byte << p);  /* Manipulate buffer */
		current_read_byte = pgetc();    /* Change read bytes */
		rv |= (current_read_byte >> (8-p));
		read_position = 7-p;
		return(rv & lmask[n]);          /* Can return pending residual val */
		}
		current_read_byte = (current_read_byte << 8) | pgetc();
		read_position += 8;                 /* else shift in new information */
		p -= 8;
	 }
  if(!p)                                 /* If position is zero */
	 {
		read_position = -1;                 /* Can return current byte */
		return(current_read_byte & lmask[n]);
	 }
  p = -p;                                 /* Else reverse position and shift */
  read_position = p-1;
  return((current_read_byte >> p) & lmask[n]);
}

// lecture d'un octet dans le buffer de code

unsigned char pgetc(void)
{
unsigned char c;

	c=bufcomp[i_lec++];
	return(c);
}

// criture d'un octet dans le buffer de code

void pputc(unsigned char c)
{
	bufcomp[i_ecr++]=c;
}


