/**********************************************************************/
/*                          W L F F T . C                              */
/*--------------------------------------------------------------------*/
/*    Fonction       :  Calcul de la fft d'un signal entr sous forme */
/*                      de tableau.                                   */
/*--------------------------------------------------------------------*/
/*    Auteur         :  MESNAGER Gilles				      				*/
/*    Dvelopp le   :  20-02-1992                                    */
/*    Dernire modif.:  16-06-1997				      						*/
/**********************************************************************/

#include <math.h>
#include <stdlib.h>
#include "wlfft.h"

#define PI 3.141592654

/*----------------------------------------------------------------------*/

void far inputfunction(int N, int *nu_ptr, complex *x, scomplex *val);
void far fastfourier(int N, int nu, complex *x,int sens);
void far output(int N, scomplex *out1, complex *x);
void far scaleinput(int N, complex *x);

/*----------------------------------------------------------------------*/

int far fft(int N, scomplex *in, scomplex *out)
{
  complex *x;
  int    nu;

  if ((x=(complex *)calloc(N, sizeof(complex)))== NULL) {
	 //fprintf(stderr, "erreur d'allocation memoire sur x.\n");
	 return(1);
  }
  inputfunction(N, &nu, x, in);
  fastfourier(N, nu, x, 1);
  output(N, out, x);
  free(x);
  return(0);
}

/*----------------------------------------------------------------------*/

int far ifft(int N, scomplex *in, scomplex *out)
{
  complex *x;
  int    nu;

  if ((x=(complex *)calloc(N, sizeof(complex)))== NULL) {
	 //fprintf(stderr, "erreur d'allocation memoire sur x.\n");
	 return(1);
  }

  inputfunction(N, &nu, x, in);
  scaleinput(N, x) ;  /* Si fft inverse uniquement */
  fastfourier(N, nu, x, -1);
  output(N, out, x);
  free(x);
  return(0);
}

/*----------------------------------------------------------------------*/

void far inputfunction(int N, int *nu_ptr, complex *x, scomplex *val)
{
  int i;
  float tmp;

  tmp = (log((float)(N))/log(2.0));
  *nu_ptr = (int)(tmp) ;
  for (i=0; i<=N-1; i++){
    x[i].re = (double)val[i].re ;
    x[i].im = (double)val[i].im ;
  }
}

/*----------------------------------------------------------------------*/

int far p2(int x)
{
  int tamp;

  switch (x) {
    case 0  : tamp = 1; break;
    case 1  : tamp = 2; break;
	 case 2  : tamp = 4; break;
	 case 3  : tamp = 8; break;
    case 4  : tamp = 16; break;
    case 5  : tamp = 32; break;
    case 6  : tamp = 64; break;
    case 7  : tamp = 128; break;
    case 8  : tamp = 256; break;
    case 9  : tamp = 512; break;
    case 10 : tamp = 1024; break;
    case 11 : tamp = 2048; break;
    case 12 : tamp = 4096; break;
  }
  return(tamp);
}

/*--------------------------------------------------------------------- */

int far bitrev(int j, int nu)
{
  int i,m,j2,k;

  m=j;
  k=0;
  for (i=1; i <= nu; i++) {
    j2 = m/2;
    k  = k*2+(m-2*j2);
    m  = j2;
  }
    return(k);
}

/*----------------------------------------------------------------------*/

void far fastfourier(int N, int nu, complex *x,int sens)
{
  int p,l,n2,nu1,i,k;
  double arg,c,s;
  complex t;

  n2 = N/2;
  nu1 = nu-1;
  k = 0;
  for (l=1; l <= nu; l++){
    do{
      for (i=1; i <= n2; i++){
	p=bitrev(k/p2(nu1),nu);
	arg=sens*2*PI*p/N;  /* sens=1 direct sens=-1 inverse */
	c=cos(arg);
	s=sin(arg);
	t.re=x[k+n2].re*c+x[k+n2].im*s;
	t.im=x[k+n2].im*c-x[k+n2].re*s;
	x[k+n2].re=x[k].re-t.re;
	x[k+n2].im=x[k].im-t.im;
	x[k].re=x[k].re+t.re;
	x[k].im=x[k].im+t.im;
	k=k+1;
		};
		k = k+n2;
    }while (k<N);
    n2 = n2/2;
    nu1 = nu1 -1;
    k = 0;
  };
  k = 0;
  while (k<N) {
    i=bitrev(k,nu);
    if (i>k) {
      t.re = x[k].re;
      t.im = x[k].im;
      x[k].re = x[i].re;
      x[k].im = x[i].im;
      x[i].re = t.re;
      x[i].im = t.im;
    }
    k = k+1;
  }
}

/*----------------------------------------------------------------------*/

void far output(int N, scomplex *out1, complex *x)
{
  int i ;

  for (i = 0; i<= N-1; i++){
    out1[i].re = (float)x[i].re  ;
    out1[i].im = (float)x[i].im  ;
  }
}

/*----------------------------------------------------------------------*/

void far scaleinput(int N, complex *x)
{
int i;

for (i=0; i<=N-1; i++){
	  x[i].re=x[i].re/N;
     x[i].im=x[i].im/N;
   }
}

/*-------------------------------------------------------------------------*/

/* Bibliothque de calcul sur des complexes */

float far re_m(scomplex x)
{
    return(x.re);
}

float far im_m(scomplex x)
{
	 return(x.im);
}

scomplex far adf_m( scomplex x,scomplex y )
{
scomplex tmp;

	 tmp.re = x.re + y.re;
	 tmp.im = x.im + y.im;
	 return(tmp);
}

scomplex far sousf_m( scomplex x,scomplex y )
{
scomplex tmp;

	 tmp.re = x.re - y.re;
    tmp.im = x.im - y.im;
    return(tmp);
}

scomplex far multcf_m( scomplex x,scomplex y )
{
scomplex tmp;

    tmp.re = x.re*y.re - x.im*y.im;
    tmp.im = x.re*y.im + x.im*y.re;
    return(tmp);
}

scomplex far divcf_m( scomplex x,scomplex y )
{
scomplex tmp;

    tmp = multcf_m(x,conjf_m(y));
    tmp = divrf_m(tmp,(modf_m(y)*modf_m(y)));
	 return(tmp);
}

scomplex far mulf_m( scomplex x,float a )
{
scomplex tmp;

	 tmp.re = x.re*a;
	 tmp.im = x.im*a;
	 return(tmp);
}

scomplex far divrf_m(scomplex x,float a)
{
scomplex tmp;

	 tmp.re = x.re/a;
	 tmp.im = x.im/a;
	 return(tmp);
}

float far modf_m(scomplex x)

{
	 return((float) sqrt(x.re * x.re + x.im * x.im) );
}

float far modcf_m(scomplex x)

{
    return(x.re * x.re + x.im * x.im);
}

scomplex far conjf_m(scomplex x)
{
	 x.im *= -1;
    return(x);
}

/*-------------------------------------------------------------------------*/
