/**********************************************************************/
/*                          W L M O D I F . C P P                     */
/*--------------------------------------------------------------------*/
/*    Fonction       : Fournit diffrentes fonctions pour le          */
/*                     traitement d'image                             */
/*--------------------------------------------------------------------*/
/*    Auteur         : MC                  				 	      	*/
/*    Dvelopp le   : 18/06/1991                                     */
/*    Dernire modif.: 16/06/1997                                     */
/**********************************************************************/

#include <math.h>

#include "wgttsi.h"
#include "wlmodif.h"

/*----------------------------------------------*/
/*																*/
/*		Constantes du programme     				   */
/*																*/
/*----------------------------------------------*/

/*----------------------------------------------*/
/*																*/
/*		Variables du programme        				*/
/*																*/
/*----------------------------------------------*/

unsigned int lutv[256];		/* LUT verte */
unsigned int lutr[256];		/* LUT rouge */
unsigned int lutb[256];		/* LUT bleue */

/*----------------------------------------------*/
/*															  	*/
/*		Variables externes  								*/
/*																*/
/*----------------------------------------------*/

/*----------------------------------------------*/
/*																*/
/*  	     Fonctions prototypes						*/
/*																*/
/*----------------------------------------------*/

/*-------------------------------------------------------------------------*/
/*                                                                         */
/*                         Procdures du programme                         */
/*       								   													*/
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
/*  Fonction :                   M A S K                                   */
/*-------------------------------------------------------------------------*/
/*  Fonction : Masquage d'image                                            */
/*  Paramtres en entre : msk = Masque  appliquer                        */
/*  Valeur retourne : Aucune                                              */
/*-------------------------------------------------------------------------*/

void far Mask(unsigned char msk,unsigned char far *ptsource,unsigned char far *ptdestin)

{

unsigned char far *pointim1,*pointim2;
unsigned int i=0;

	if(ptsource==0) ptsource=source;
	if(ptdestin==0) ptdestin=destin;

	pointim1=ptsource;
	pointim2=ptdestin;

	do
	{
	*pointim2++=(*pointim1++)&msk;
	}
	while(++i);
}

/*-------------------------------------------------------------------------*/
/*  Fonction :                   M O D H I S T                             */
/*-------------------------------------------------------------------------*/
/*  Fonction : Calcul de table et modification d'histogramme               */
/*  Paramtres en entre : Type de modif, Table ,bufhist, bufrepa          */
/*  Valeur retourne : Aucune                                              */
/*-------------------------------------------------------------------------*/

void far Modhist(char c,unsigned int tab0[256],
		  unsigned int bufhist[256],unsigned int bufrepa[256],
	unsigned char far *ptsource,unsigned char far *ptdestin)
{
unsigned char far *pointim1,*pointim;   	/* pointeurs image */
unsigned int i=0;
unsigned char a;

	if(ptsource==0) ptsource=source;
	if(ptdestin==0) ptdestin=destin;
	Caltab(c,tab0);
	Calhist(bufhist,bufrepa,ptsource);

	pointim=ptsource;
	pointim1=ptdestin;

		do
		{
		a=*pointim++;
		a=(bufrepa[a]>>8);
		a=(tab0[a]>>8);
		*pointim1++=a;
		}
		while(++i);
}

/*-------------------------------------------------------------------------*/
/*  Fonction :                   C A L T A B                               */
/*-------------------------------------------------------------------------*/
/*  Fonction : Calcul de table pour modification d'histogramme             */
/*  Paramtres en entre : Type de modif, Table                            */
/*  Valeur retourne : Aucune                                              */
/*-------------------------------------------------------------------------*/

void far Caltab(char c,unsigned int tab0[256])
{
unsigned long a;
unsigned int i;
double d;
int b;

	switch(c)
	{

	case '0':     		/* linaire */

	for(i=0;i<256;i++)  {tab0[i]=i<<8;} break;

	case '1':               /* montant */

	for(i=0;i<256;i++)
	{
	b=i*100;
	d=sqrt((float)b);
	a=410*((long)d);
	tab0[i]=(unsigned int)a;
	}
	break;

	case '2':           	/* descendant */

	for(i=0;i<256;i++)
	{
	b=(255-i)*100;
	d=sqrt((float)b);
	a=410*((long)d);
	tab0[i]=(unsigned int)(65535L-a);
	}
	break;

	case '3':         	/* descendant-montant */

	for(i=0;i<128;i++)
	{
	b=(127-i)*100;
	d=sqrt((float)b);
	a=290*((long)d);
	tab0[i]=(unsigned int)(32767L-a);
	}
	for(i=0;i<128;i++)
	{
	b=i*100;
	d=sqrt((float)b);
	a=290*((long)d);
	tab0[128+i]=(unsigned int)(32768L+a);
	}
	break;

	case '4':			/* montant-descendant */

	for(i=0;i<128;i++)
	{
	b=i*100;
	d=sqrt((float)b);
	a=290*((long)d);
	tab0[i]=(unsigned int)a;
	}
	for(i=0;i<128;i++)
	{
	b=(127-i)*100;
	d=sqrt((float)b);
	a=290*((long)d);
	tab0[128+i]=(unsigned int)(65535L-a);
	}
	break;
	}
}

/*-------------------------------------------------------------------------*/
/*  Fonction :                    P S E U D O                              */
/*-------------------------------------------------------------------------*/
/*  Fonction : Pseudo couleur                                              */
/*  Paramtres en entre : Aucun                                           */
/*  Valeur retourne : Aucune                                              */
/*-------------------------------------------------------------------------*/

void far Pseudo(void)
{
unsigned int r,v,b,a;
int x,y;
unsigned char far *pointim3,*pointim1,*pointim2,*pointim; /* pointeurs image */

	pointim3=image3;
	pointim1=image1;
	pointim2=image2;

	/* chelle de gris */

	for(y=0;y<256;y++)
		 {
	    for(x=248;x<256;x++)
		{
		pointim=source+((y<<8)+x);
		*pointim=y;
		}
	    }

	pointim=source;

	for(y=0;y<256;y++)
	    {
		 for(x=0;x<256;x++)
		{
		a=*pointim++;
		v=lutv[a]>>8;
		r=lutr[a]>>8;
		b=lutb[a]>>8;
		*pointim3++=v;
		*pointim1++=b;
		*pointim2++=r;
		}
		 }
}

/*-------------------------------------------------------------------------*/
/*  Fonction :                    C A L L U T                              */
/*-------------------------------------------------------------------------*/
/*  Fonction : Calcul LUTS                                                 */
/*  Paramtres en entre : choix table, seuils                             */
/*  Valeur retourne : Aucune                                              */
/*-------------------------------------------------------------------------*/

void far Callut(char c,int seuinf,int seusup)
{
float f;
unsigned int i,tabv[256],tabr[256],tabb[256];

	switch(c)
	{
	case '1':
		for(i=0;i<51;i++)
		{
		tabv[i]=(i)<<8;
		f=256*(8+(i*(144-8)/51));
		tabr[i]=(unsigned int)f;
		f=256*(66+(i*(44-66)/51));
		tabb[i]=(unsigned int)f;
		}
		for(i=51;i<102;i++)
		{
		tabv[i]=(i)<<8;
		f=256*(144+((i-51)*(180-144)/51));
		tabr[i]=(unsigned int)f;
		f=256*(44+((i-51)*(164-44)/51));
		tabb[i]=(unsigned int)f;
		}
		for(i=102;i<153;i++)
		{
		tabv[i]=(i)<<8;
		f=256*(180+((i-102)*(20-180)/51));
		tabr[i]=(unsigned int)f;
		f=256*(164+((i-102)*(244-164)/51));
		tabb[i]=(unsigned int)f;
		}
		for(i=153;i<204;i++)
		{
		tabv[i]=(i)<<8;
		f=256*(20+((i-153)*(84-20)/51));
		tabr[i]=(unsigned int)f;
		f=256*(244+((i-153)*(52-244)/51));
		tabb[i]=(unsigned int)f;
		}
		for(i=204;i<256;i++)
		{
		tabv[i]=(i)<<8;
		f=256*(84+((i-204)*(255-84)/51));
		tabr[i]=(unsigned int)f;
		f=256*(52+((i-204)*(20-52)/51));
		tabb[i]=(unsigned int)f;
		}
		break;

	case '2':
		for(i=0;i<128;i++)
		{
		tabv[i]=i<<9;
		tabr[i]=0;
		tabb[i]=(254-(i<<1))<<8;
		}
		for(i=128;i<256;i++)
		{
		tabv[i]=(510-(i<<1))<<8;
		tabr[i]=(i-128)<<9;
		tabb[i]=0;
		}
		break;

	case '3':
		for(i=0;i<128;i++)
		{
		tabv[i]=0;
		tabr[i]=i<<9;
		tabb[i]=(254-(i<<1))<<8;
		}
		for(i=128;i<256;i++)
		{
		tabv[i]=(i-128)<<9;
		tabr[i]=(510-(i<<1))<<8;
		tabb[i]=0;
		}
		break;

	case '4':
		for(i=0;i<128;i++)
		{
		tabv[i]=0;
		tabr[i]=(254-(i<<1))<<8;
		tabb[i]=i<<9;
		}
		for(i=128;i<256;i++)
		{
		tabv[i]=(i-128)<<9;
		tabr[i]=0;
		tabb[i]=(510-(i<<1))<<8;
		}
		break;
	}

	if(c>='0'&&c<='4')
	{
	for(i=0;i<256;i++) {lutv[i]=lutr[i]=lutb[i]=i<<8;}

	 if(seuinf!=seusup)
	 {
		for(i=seuinf;i<=seusup;i++)
		{
		lutv[i]=tabv[((i-seuinf)*255)/(seusup-seuinf)];
		lutr[i]=tabr[((i-seuinf)*255)/(seusup-seuinf)];
		lutb[i]=tabb[((i-seuinf)*255)/(seusup-seuinf)];
		}
	 }
	 else
	 {
		i=seuinf;
		lutv[i]=tabv[0];
		lutr[i]=tabr[0];
		lutb[i]=tabb[0];
	 }

	}
}

/*-------------------------------------------------------------------------*/
/*  Fonction :                   P L A N B I T                             */
/*-------------------------------------------------------------------------*/
/*  Fonction : Plan de bit (0-7) d'image                                   */
/*  Paramtres en entre : msk = Masque  appliquer                        */
/*  Valeur retourne : Aucune                                              */
/*-------------------------------------------------------------------------*/

void far Planbit(unsigned char msk,unsigned char far *ptsource,
		unsigned char far *ptdestin)
{
unsigned char far *pointim1,*pointim2;
unsigned char p;
unsigned int i=0;

	if(ptsource==0) ptsource=source;
	if(ptdestin==0) ptdestin=destin;
	pointim1=ptsource;
	pointim2=ptdestin;

	do
	{
	p=((*pointim1++)&msk);
	if(p==0) *pointim2++=0;else *pointim2++=255;
	}
	while(++i);

}

/*-------------------------------------------------------------------------*/

void far Testres(char c,unsigned char far *ptsource,unsigned char far *ptdestin)
{
unsigned char far *pointim1,*pointim2;
int x,y,msk;

	if(ptsource==0) ptsource=source;
	if(ptdestin==0) ptdestin=destin;

				switch(c)
				{
				case '0': msk=0xff;break;
				case '1': msk=0xfe;break;
				case '2': msk=0xfc;break;
				case '3': msk=0xf8;break;
				case '4': msk=0xf0;break;
				}

		pointim2=ptdestin;
		for(y=0;y<256;y++)
		{
				for(x=0;x<256;x++)
				{
				pointim1=ptsource+((y&msk)<<8)+(x&msk);
				*pointim2++=*pointim1;
				}
		}
}

/*-------------------------------------------------------------------------*/

	/* Echelle de gris */

void far Echelle(unsigned char far *image)
{

int x,y;
unsigned char far * pointim;

	for(y=0;y<256;y++)
	 {
	    for(x=248;x<256;x++)
			{
			pointim=image+((y<<8)+x);
			*pointim=y;
			}
	  }
}

/*-------------------------------------------------------------------------*/
