/**********************************************************************/
/*                      W L S E G . C 	                              */
/*--------------------------------------------------------------------*/
/*    Fonction       : Fournit diffrentes fonctions pour la          */
/*                     segmentation d'images                          */
/*--------------------------------------------------------------------*/
/*    Auteurs        : MC		                			               */
/*    Dvelopp le   : 22/12/1995                                     */
/*    Dernire modif.: 16/06/1997                                     */
/**********************************************************************/

#include <math.h>

#include "wgttsi.h"

#include "wlseg.h"

/*----------------------------------------------*/
/*																*/
/*		Constantes du programme         				*/
/*																*/
/*----------------------------------------------*/

/*----------------------------------------------*/
/*						*/
/*		Variables du programme          */
/*						*/
/*----------------------------------------------*/

/*----------------------------------------------*/
/*						*/
/*		Variables externes  		*/
/*						*/
/*----------------------------------------------*/

/*----------------------------------------------*/
/*						*/
/*  	     Fonctions prototypes		*/
/*						*/
/*----------------------------------------------*/

/*-------------------------------------------------------------------------*/
/*                                                                         */
/*                         Procdures du programme                         */
/*       								   													*/
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
/*  Fonction :                    S E G _ H I S C O U                      */
/*-------------------------------------------------------------------------*/
/*  Fonction : Segmentation sur image rduite                              */
/*  Paramtres en entre : tiquette de la classe                          */
/*  Valeur retourne : Aucune                                              */
/*-------------------------------------------------------------------------*/

void far Seg_hiscou(int a)

{

unsigned char r,v,p;
int x,y;
unsigned char far *pointim3,*pointim1,*pointim2,*pointim0;

	pointim3=image3;
	pointim1=image1;
	pointim2=image2;

	for(y=0;y<256;y++)
	    {
	    for(x=0;x<256;x++)
		{
		v=*pointim3;
		r=*pointim2;
		pointim0=image0+r+(v<<8);
		p=*pointim0;
		if(p!=(unsigned char)(256-a))
			{
			*pointim3++=0;
			*pointim1++=0;
			*pointim2++=0;
			}
		else
			{
			pointim3++;pointim2++;pointim1++;
			}
		}
	    }
}


/*-------------------------------------------------------------------------*/
/*  Fonction :                    S E G C O U                              */
/*-------------------------------------------------------------------------*/
/*  Fonction : Segmentation couleur ; rsultat dans plan 0                 */
/*  Paramtres en entre : min, max des 3 composantes                      */
/*  Valeur retourne : Aucune                                              */
/*-------------------------------------------------------------------------*/

void far Segcou(int vmin,int vmax,int rmin,int rmax,int bmin,int bmax)

{

int r,v,b,x,y;
unsigned char far *pointim3,*pointim2,*pointim1,*pointim0; /* pointeurs image */

	pointim3=image3;
	pointim2=image2;
	pointim1=image1;
	pointim0=image0;

	for(y=0;y<256;y++)
		 {
	    for(x=0;x<256;x++)
		{
		v=*pointim3++;
		r=*pointim2++;
		b=*pointim1++;
		if((vmax>=v)&&(v>=vmin)&&(rmax>=r)&&(r>=rmin)
		&&(bmax>=b)&&(b>=bmin))
		*pointim0++=255;else *pointim0++=0;
		}
	     }
}

/*-------------------------------------------------------------------------*/
/*  Fonction :                    R E D U I                                */
/*-------------------------------------------------------------------------*/
/*  Fonction : Transformation en coordonnes rduites                      */
/*  Paramtres en entre : Aucun                                           */
/*  Valeur retourne : Aucune                                              */
/*-------------------------------------------------------------------------*/

void far Redui(int seuil)

{

unsigned int r,v,b,s;
int x,y;
unsigned char far *pointim3,*pointim1,*pointim2; /* pointeurs image */

	pointim3=image3;
	pointim1=image1;
	pointim2=image2;

	for(y=0;y<256;y++)
	    {
	    for(x=0;x<256;x++)
		{
		v=*pointim3;
		b=*pointim1;
		r=*pointim2;
		s=r+v+b;
		if(s>0&&s>seuil)
			{
			v<<=8;
			b<<=8;
			r<<=8;
			*pointim3++=v/s;
			*pointim1++=b/s;
			*pointim2++=r/s;
			}
		else
			{
			*pointim3++=0;
			*pointim1++=0;
			*pointim2++=0;
			}
		}
	    }
}

/*-------------------------------------------------------------------------*/
/*  Fonction :                     I T S                                   */
/*-------------------------------------------------------------------------*/
/*  Fonction : Transformation en intensit-teinte-saturation               */
/*  Paramtres en entre : Aucun                                           */
/*  Valeur retourne : Aucune                                              */
/*-------------------------------------------------------------------------*/

void far Its(int seuil)

{

int x,y;
unsigned int r,v,b,s,intens,teint,sat,min,a;
unsigned char far *pointim3,*pointim1,*pointim2;  /* pointeurs image */

	pointim3=image3;
	pointim1=image1;
	pointim2=image2;

	for(y=0;y<256;y++)
		 {
		 for(x=0;x<256;x++)
		{
		v=*pointim3;
		b=*pointim1;
		r=*pointim2;
		s=r+v+b;
		if(s>0&&s>seuil)
			{
			intens=((s<<6)/3)>>6;
			v<<=8;
			b<<=8;
			r<<=8;
			v/=s;
			b/=s;
			r/=s;
			min=v;
			if(min>r)min=r;
			if(min>b)min=b;
			sat=255-(3*min);
			a=3*sat;
			if(a!=0)
			{
			if(min==b){teint=((v-b)<<8)/a;}
			else {
				if(min==r){teint=85+((b-r)<<8)/a;}
				else teint=170+((r-v)<<8)/a;
				  }
			}
			else teint=0;
			*pointim1++=intens;
			*pointim2++=sat;
			*pointim3++=teint;
			}
		else
			{
			*pointim3++=0;
			*pointim1++=0;
			*pointim2++=0;
			}
		}
	    }
}

/*-------------------------------------------------------------------------*/
/*  Fonction       : 		H O U G H D I R		                   */
/*-------------------------------------------------------------------------*/
/*  Fonction : Transforme de HOUGH directe	                           */
/*  Paramtres en entre : Aucun	                                   */
/*  Valeur retourne : Aucune                                              */
/*-------------------------------------------------------------------------*/


void far Houghdir(unsigned char far *ptsource,unsigned char far *ptdestin)

{

float a=0.4158;                        /* Facteur d'chelle */
float c;
int x,y,i,j;
unsigned char *pix_source,*pix_destin;
int tab_cos[256],tab_sin[256];

	if(ptsource==0) ptsource=source;
	if(ptdestin==0) ptdestin=destin;

Eff(ptdestin);

for (i=0;i<=255;i++)
	{
	c=(float)cos((double)(0.0061359+i*0.0122718));
   if(c>=0) 	tab_cos[i]=(int)(256*a*c+0.5);
   else 	tab_cos[i]=(int)(256*a*c-0.5);

   c=(float)sin((double)(0.0061359+i*0.0122718));
   if(c>=0) 	tab_sin[i]=(int)(256*a*c+0.5);
	else 	tab_sin[i]=(int)(256*a*c-0.5);
   }

pix_source=ptsource;

for(y=0;y<=255;y++)
	{
	for(x=0;x<=255;x++)
		{
		if(*pix_source>=128)
		{
		for(i=0;i<=255;i++)
			{
			j=x*tab_cos[i]+y*tab_sin[i]+27146;  /* 106.038*256 */
			pix_destin=ptdestin+i+(j&0XFF00);

			if(*pix_destin<255) *pix_destin+=1;
			}
		}
		pix_source++;
		}
	}
}

/*-------------------------------------------------------------------------*/
/*  Fonction       : 		H O U G H I N V		                   			*/
/*-------------------------------------------------------------------------*/
/*  Fonction : Transforme de HOUGH inverse	                           	*/
/*  Paramtres en entre : Seuil sur plan de HOUGH                         */
/*  Valeur retourne : Aucune                                              */
/*-------------------------------------------------------------------------*/

void far Houghinv(int seuil,unsigned char far *ptsource,unsigned char far *ptdestin)

{

float aa=0.4158;     	/* 1/(1+RAC2)	*/
float bb=106.038;
float d0=0.0122718;     /* PI/256 	*/

unsigned int xmin,xmax,ymin,ymax,i,j;
int a,b,c,d,n;
unsigned char far *pix_source;
float rho,t0,c0,s0,alpha,beta,gamma,delta;

	if(ptsource==0) ptsource=source;
	if(ptdestin==0) ptdestin=destin;

Eff(ptdestin);
pix_source=ptsource;

for(j=0;j<=255;j++)
	{
	for(i=0;i<=255;i++)
		{
	 if(*pix_source>=seuil)
	 {
	 rho=(j-bb)/aa;
	 t0=(i+0.5)*d0;
	 c0=(float)cos((double)t0);
	 s0=(float)sin((double)t0);

	 alpha=rho/s0;
	 beta=rho/c0;
	 gamma=(rho-255*c0)/s0;
	 delta=(rho-255*s0)/c0;
	 a=b=c=d=0;

	 if((0<=alpha) && (alpha<255))	a=1;
	 if((0<beta)   && (beta<=255)) 	b=1;
	 if((0<gamma)  && (gamma<=255))	c=1;
	 if((0<=delta) && (delta<255)) 	d=1;

	 n=(a<<3)+(b<<2)+(c<<1)+d;

	 xmin=ymin=0;
	 xmax=ymax=255;

	 switch(n)
	 {
	  case 1 :  ymin = 255;break;
	  case 2 :  xmin = 255;break;
	  case 3 :  xmin = delta;ymin=255;ymax=gamma;break;
	  case 4 :  ymax = 0;break;
	  case 5 :  xmin = beta;xmax=delta;break;
	  case 6 :  xmin = beta;ymax=gamma;break;
	  case 7 :  xmin = 255;break;
	  case 8 :  xmax = 0;break;
	  case 9 :  ymin = alpha;xmax=delta;break;
	  case 10 : ymin = alpha;ymax=gamma;break;
	  case 11 : ymin = 255;break;
	  case 12 : ymin = alpha;xmax=beta;ymax=0;break;
	  case 13 : xmax = 0;break;
	  case 14 : ymax = 0;break;
	 }

	 if((n==15)&&(alpha>250))        /* cas de la diagonale secondaire */
		 {
		 ymin=255;ymax= 0;
		 }

	 if(n!=0) Droite(xmin,ymin,xmax,ymax,ptdestin,255);
	 }
	 pix_source++;
		}
	}
}

/*-------------------------------------------------------------------------*/
