/***************************************************************/
/*                   W L S E Q N B . C P P            	      */
/*-------------------------------------------------------------*/
/*    Fonction       : Gestion de squences NB	   		   	*/
/*-------------------------------------------------------------*/
/*    Auteur         : MC                                      */
/*    Dvelopp le   : 24/04/97                                */
/*    Dernire modif.: 17/06/97                                */
/***************************************************************/

#include <dos.h>
#include <stdio.h>
#include <alloc.h>
#include <io.h>

#include "wgttsi.h"
#include "wlcomp.h"
#include "wlseqnb.h"
#include "wldecbin.h"

/*----------------------------------------------*/
/*                                              */
/*              Constantes du programme         */
/*                                              */
/*----------------------------------------------*/

#define NBIMAX 16383      // Nb max d'images de squence

/*----------------------------------------------*/
/*                                              */
/*              Variables du programme          */
/*                                              */
/*----------------------------------------------*/

unsigned char far *bufim1=NULL;  // buffer de codes
long nbimage1;                   // numro d'image courante
FILE *in1=0;                      // Fichier squence
en_tete_image en_tete_im;         // en-tte
unsigned int taille_en_tete;      // taille de l'en-tte
chaine_seqnb chaine;              // chanage
caracseqnb carseqnb;              // caractristiques de la squence
long far *tab_addebim=NULL;     // adresses de dbut des images
//struct ftime tc3;            // date & heure

/*----------------------------------------------*/
/*                                              */
/*              Variables externes              */
/*                                              */
/*----------------------------------------------*/

/*----------------------------------------------*/
/*                                              */
/*           Fonctions prototypes               */
/*                                              */
/*----------------------------------------------*/

void far Init_debut1(FILE *in);
void far Avance(FILE *in,int mode);
void far Recule(FILE *in,int mode);
void far Lect_im(FILE *in,int mode);

/*-------------------------------------------------------------*/
/*                                                             */
/*                   Procdures du programme                   */
/*                                                             */
/*-------------------------------------------------------------*/

unsigned int far Code_nb(en_tete_image * en_tete,unsigned char far *image,
		unsigned char *tabcode,int seuil,unsigned char far *image_seuil)
{
unsigned int ret=0;
char type;

	type=en_tete->type_codage;

	gettime(&en_tete->timimage);
	getdate(&en_tete->dateimage);

	switch(type)
	{
	case 0 :
				Copie_plan(image,tabcode);
				break;
	case 1 :
	case 2 :
				ret=Code_dct(type,en_tete->qualite,image,tabcode);
				break;
	case 3 :
				ret=Codbin(tabcode,image,en_tete->xmin,en_tete->xmax,
				en_tete->ymin,en_tete->ymax,seuil,image_seuil);
				break;
	}

	en_tete->nbcodim=ret;

	return(ret);
}

/*-------------------------------------------------------------------------*/

void far Decode_nb(en_tete_image * en_tete,unsigned char far *image,
				unsigned char *tabcode)
{
char type;

	type=en_tete->type_codage;

	switch(type)
	{
	case 0 :
				Copie_plan(tabcode,image);
				break;
	case 1 :
	case 2 :
				Decode_dct(type,en_tete->qualite,image,tabcode);
				break;
	case 3 :
				Syntbin(en_tete->nbcodim,tabcode,image);
				break;
	}
}

/*-------------------------------------------------------------------------*/

void far Init_debut1(FILE *in)
{
	if(!bufim1) bufim1=(unsigned char *)farmalloc(65536L);     // allocation
	if(!tab_addebim) tab_addebim=(long *)farmalloc(65536L);     // allocation

	nbimage1=0;               // lecture 1re image
	rewind(in);
	fread(&chaine,1,sizeof(chaine_seqnb),in);
	if(chaine.suiv!=0) Lect_im(in,0);
}

/*-------------------------------------------------------------------------*/

void far Lect_im(FILE *in,int mode)
{
long l;

	if(mode==0)   // on lit
	{
	fread(&taille_en_tete,2,1,in);
	fseek(in,-2L,SEEK_CUR);
	fread(&en_tete_im,taille_en_tete,1,in);
	l=chaine.suiv-taille_en_tete-8;
	if(l<65528L) fread(bufim1,(unsigned int)l,1,in);
	else{fread(bufim1,8,1,in);fread(bufim1+8,(unsigned int)(l-8),1,in);}
	}
	else fseek(in,chaine.suiv-8,SEEK_CUR);         // 8 = 2*sizeof(long) // chaine
}

/*-------------------------------------------------------------------------*/

void far Avance(FILE *in,int mode)
{

	fread(&chaine,1,sizeof(chaine_seqnb),in);
	if(chaine.suiv==0)
	{Init_debut1(in);return;} // retour premire
	Lect_im(in,mode);
	nbimage1++;
}

/*-------------------------------------------------------------------------*/

void far Recule(FILE *in,int mode)
{
long offset;

	if(chaine.prec==0) return;
	fseek(in,-chaine.suiv,SEEK_CUR);
	fread(&chaine,1,sizeof(chaine_seqnb),in);

	if(chaine.prec==0)
	{
	Lect_im(in,0);
	return;
	} // retour premire
	offset=(long)(chaine.prec+8);
	fseek(in,-offset,SEEK_CUR);

	fread(&chaine,1,sizeof(chaine_seqnb),in);
	Lect_im(in,mode);
	nbimage1--;
}

/*-------------------------------------------------------------------------*/
/*  Fonction :                    A N I M A S E Q                         */
/*-------------------------------------------------------------------------*/
/*  Fonction : Animation d'images  codes         			                 */
/*  Paramtres en entre : Fichier,numero, plan           						*/
/*  Valeur retourne : nb de codes de l'image    	     					      */
/*-------------------------------------------------------------------------*/

long far Animaseq(FILE *in,long numero,unsigned char far *image)
{
long diff;
long dep;

	if(numero<=NBIMAX-1&&nbimage1<=NBIMAX-1)
	{
	dep=tab_addebim[(int)numero]-tab_addebim[(int)nbimage1]-chaine.suiv;
	fseek(in,dep,SEEK_CUR);
	fread(&chaine,1,sizeof(chaine_seqnb),in);
	Lect_im(in,0);
	nbimage1=numero;
	}
	else
	{
	diff=nbimage1-numero;
	if(diff<0)
	{
		while(diff!=-1)
		{
		Avance(in,1);
		diff=nbimage1-numero;
		}
		Avance(in,0);
	}

	if(nbimage1>numero)
	{
		while(diff!=1)
		{
		Recule(in,1);
		diff=nbimage1-numero;
		}
		Recule(in,0);
	}
	}
	if(image!=NULL) Decode_nb(&en_tete_im,image,bufim1);
	return(chaine.suiv);
}

/*-------------------------------------------------------------------------*/
/*  Fonction :                    C A R S E Q N B                          */
/*-------------------------------------------------------------------------*/
/*  Fonction : Dtermination des carctristiques de squences NB			   */
/*  Paramtres en entre : Fichier  ,structure      							   */
/*  Valeur retourne : aucune			 		        	           					*/
/*-------------------------------------------------------------------------*/

void far Carseqnb(FILE *in,caracseqnb *carseqnb)
{
long nbtot=0,totcod;
long offset;

	Init_debut1(in);      // init pour squence nb
	tab_addebim[0]=0;
	totcod=chaine.suiv;
	while(chaine.suiv)
	{
	nbtot++;
	if(nbtot<=NBIMAX-1) tab_addebim[(int)nbtot]=totcod;
	fread(&chaine,1,sizeof(chaine_seqnb),in);
	totcod+=sizeof(chaine_seqnb);

	if(chaine.suiv==0)
	{
	Init_debut1(in);
	break;
	} // retour premire
	offset=chaine.suiv-8;
	fseek(in,offset,SEEK_CUR);     // on saute l'image
	totcod+=offset;
	}
	carseqnb->nbtot=nbtot;
	carseqnb->totcod=totcod;
	if(nbtot) carseqnb->codmoyen=(float)totcod/(float)nbtot;
	else carseqnb->codmoyen=(float)totcod;
}

/*-------------------------------------------------------------------------*/
