#include "myowlall.h"
#include "baptiall.h"

#pragma hdrstop

#define MODULE_NAME "ANLFREQ.CPP"
 /*******************|******************************************
                      W A N L F R E Q . C
    -------------------------------------------------------------
        Fonction       : Fonction de traitement d'image pour
                         Anagnostes.
    
     Fichier source WANLFREQ.C.
     (contient les objets internes
      et les corps des objets  exporter)
    -------------------------------------------------------------
        Auteur         : Baptise MARCEL
        Dvelopp le   : printemps 1996
    *************************************************************
    
    // Inclusions bibliothques standards
    #include <math.h>
    #include <stdlib.h>
    #include <values.h>
    // Inclusions bibliothques Object Windows
    #include <owl/point.h>
    // Inclusions bibliothques DOS
    // Inclusion bibliothques Image
    */ 

#include "wgttsi.h"

//  Inclusions bibliothques Iris
#include "anlfreq.h"
#include "base.h"
#include "sysctr.h"
#include "wgeometr.h"
#include "piadiv.h"
#include "scaltool.h"

/*-------------------------------------------------------------
                        constantes locales
   -------------------------------------------------------------*/
#define BALAYAGE_EN_COLONNE 0
#define BALAYAGE_EN_LIGNE 1
 /*-------------------------------------------------------------
                        variables  exporter
    -------------------------------------------------------------*/ 

float VN_pof = 0.25, K_pof = 1.0, VN_etn = 0.1, K_etn = 1.0, VN_nfs = 0.5
  , K_nfs = 1.0 ;

/* Elments de calcul du critre de frquence de coupe.
   -------------------------------------------------------------
                       nouveaux types locaux
   -------------------------------------------------------------
   -------------------------------------------------------------
                        variables locales
   -------------------------------------------------------------
   
   extern unsigned char *aFreq;
   extern unsigned char *aCrit;
   extern unsigned int wNbElemFreqCrit;
   
   -------------------------------------------------------------
                 prototypes des fonctions locales
   -------------------------------------------------------------*/
void  pontage_long (unsigned char *creneaux, int freq, int *Pnc) ;
void  hachage_long (unsigned char **Pcreneaux, int freq, int *Pnc) ;
float Critere_frequence (int frequence, int qpfm, float periode
  , float ect_per, int nb_fus, int nb_sep) ;
BOOL  CalculeCourbeFreqQuad (TCoordonnee AN, TCoordonnee BN
  , TCoordonnee CN, TCoordonnee DN, int iNbElements, char cSensParcours
  , char fBalayage, unsigned int SymbolReadingProperties
  , unsigned char *auTableFreq, unsigned char *auTableCrit
  , unsigned char *puSource, unsigned char uMode = ANLFREQ_FrequenceImpaire
  , float *axCellTable = NULL, TBipoint *pbipCellBearer = NULL
  , unsigned int wTrancheAAnalyser = 0) ; // Pour le cas o l'on veuille calculer les positions de cellule pour une tranche donne
char  AnalyseCourbeFreqQuad (unsigned char *auTabFreq
  , unsigned char *auTabCrit, int iNbCoupes, unsigned int *pwFreq
  , unsigned int *pwIndexFrequence = NULL, unsigned int *pwRetries = NULL) ;
BOOL  AlSeuilleCoupe (unsigned char *courbe, float seuil, int l
  , unsigned char *courbe_carree) ;
BOOL  SeuilleLigne (unsigned char *puLigne, unsigned int wLongueurLigne
  , float xValeurSeuil) ;
BOOL  AnalyseCourbeFreqQuadMinCritDeb (unsigned char *auTabFreq
  , unsigned char *auTabCrit, int iNbCoupes, unsigned int *pwFreq
  , unsigned char uIdxCritMin, unsigned int *pwIndexFrequence = NULL
  , unsigned int *pwRetries = NULL) ;
BOOL  AnalyseDestFrequenceLigne (unsigned char *auLigne
  , unsigned int uLongueurLigne, float xValeurSeuil, int *piFrequence
  , float *pxPeriode, float *pxEcartTypePeriode, unsigned int *pvPeriodeMin
  , unsigned int *pwPeriodeMax, BOOL *pRacourcissementD
  , BOOL *pRacourcissementF, float *pxSeuilRejet, BOOL *pfCompatibilite
  , unsigned int wProperties, float *axCellTable = NULL) ;

/*-------------------------------------------------------------------------*/
inline BOOL PeriodeAccidentelle (
    unsigned char uP // Longueur de la priode
    ,float xPMoy // Longueur moyenne des priodes (hors bords)
    , double xEcTypPer // cart type moyen des priodes (hors bords)
    , float *pxSeuilRejet = NULL  // Valeur du rejet (calcule ici et propage vers les appeleurs
    )
{
// Priode rejete si demi cart  la moyenne >= xSeuilRejet
float xSeuilRejet = max(1.0, 2 * xEcTypPer +0.2*xPMoy);
//                                         ^^^^^^^^^^^
/*     Ceci a t rajout plus tard, lorsque l'on s'est aperu que les grandes priodes taient souvent rejetes.
  L'influence est faible sur les petites priodes. 0 partir de priodes de taille 5, a ajoute 1px.
  Si les priodes sont grandes, les rejets sont sncs rejeter des assemblages d'un nombre entier de cellules, et donc des carts de l'ordre dexPMoy.
   
*/
/*

 xSeuilRejet vaut 2cart type, mais on accpte toute priode si l'cart  la moyenne est <= 1.0
*/

if ( pxSeuilRejet ) 
  *pxSeuilRejet =  xSeuilRejet ;
return uP < NRoundR(xPMoy - xSeuilRejet)
        || uP > NRoundR(xPMoy + xSeuilRejet);
                                                  //: Obsolete
        /*   return fabs(xPMoy-uP) // cart  la moyenne
             >= xSeuilRejet; */

/*
Note: Le principal problme est de prendre en considretion la
quantumisation (d'o l'exception 1/2).
La valeur du seuil tait fixe  max(1,xEcTypPer), mais le cas ...
a montr qu'une ligne peut tre du type (liste des longueurs de priode)
"1 3 3 2 3 3 2 3 3 3 2 3 3 2 3 3 2 3 1" o les 2 extrmits doivent tre
supprimmes.
Si ce choix ne s'averrait pas astucieux, il faut reconsidrer l'exception
1/2, rechercher les cas exacts o elle s'applique et veiller  ce qu'elle
ne soit appliqu que dans ces cas l.


Note: Autre problme: priode moyenne de 5.733333, Ec.Typ. de 0.5962863
devait autoriser des priodes jusqu' 6.9259026.
  Or, une priode de 7 tait valable dans ce cas.
  La formule prends ceci en compte:
    Avant: return fabs(xPMoy-uP)>= xSeuilRejet; // cart  la moyenne
   Aprs:
   return uP >= NRoundR(xPMoy-xSeuilRejet) &&
          uP <= NRoundR(xPMoy+xSeuilRejet)
*/
}

/*-------------------------------------------------------------------------
   -------------------------------------------------------------
                       fonctions  exporter
                     et fonctions locales
   -------------------------------------------------------------
   ----------------------------------------------------------------------
    SeuilleCoupe (Seuille coupe)
     Prend une courbe de taille "l", la seuille avec le seuil donn et renvoit
   le signal en crneaux correspondant (Cf. spcifications dans LTATOU.H).
     Le signal en crneaux renvoy est allou par la proc.
     courbe :      courbe initiale  seuiller,
     seuil :       seuil,
     l :           longueur de la courbe initiale (l<=256).
   */
unsigned char *SeuilleCoupe ( unsigned char *courbe, float seuil, int l )
{
char          niveau = 0, bas = 0, haut = 1 ;
unsigned char *courbe_carree = NULL ;
int           i = 0, n = 0 ;

if ( l < 0 ) 
  {
  MErreur("Courbe de longueur ngative, par Cthulhu!", 2385);
  return NULL ;
  }
if ( l == 0 ) 
  {
  courbe_carree =  (unsigned char *)malloc(3 * sizeof(unsigned char));
  if ( courbe_carree == NULL ) 
    {
    Erreur("Erreur d'allocation (ref. 2901)");
    return NULL ;
    }
  courbe_carree [0] =  0 ;
  courbe_carree [1] =  - 1 ;
  courbe_carree [2] =  0 ;
  return courbe_carree ;
  }
else if ( courbe == NULL ) 
  {
  Erreur("Courbe vide (ref. 2911)");
  return NULL ;
  }
for ( i = 0 ; ((float)(courbe [i]) == seuil) && (i < l) ; i++ ) 
  ;

/* Chercher le premier lment diffrent du seuil (qui dtermine le
   niveau de dpart). */
if ( i == l ) /* Pas d'lments, courbe plate gale au seuil. */ 
                {
                courbe_carree
                  =  (unsigned char *)malloc(3 * sizeof(unsigned char));
                if ( courbe_carree == NULL ) 
                  {
                  Erreur("Erreur d'allocation (ref. 2913)");
                  return NULL ;
                  }
                courbe_carree [0] =  0 ;
                courbe_carree [1] =  0 ;
                courbe_carree [2] =  l - 1 ;
                return courbe_carree ;
                }
courbe_carree =  (unsigned char *)malloc((l + 2) * sizeof(unsigned char));

// Allou avec l+2 lments, l>=1, l+2 >= 3
if ( courbe_carree == NULL ) 
  {
  Erreur("Erreur d'allocation (ref. 2920)");
  return NULL ;
  }
if ( (float)(courbe [i]) > seuil ) 
  niveau =  haut ;
else 
  niveau =  bas ;
courbe_carree [1] =  niveau ;
n                 =  2 ;
for ( i = 1 ; i < l ; i++ ) 
  if ( (niveau == haut) && ((float)(courbe [i]) < seuil) ) 
    {
    //n=i+1. i>l et n<l+1
    courbe_carree [n++] =  i - 1 ;
    niveau              =  bas ;
    }
  else if ( (niveau == bas) && ((float)(courbe [i]) > seuil) ) 
    {
    courbe_carree [n++] =  i - 1 ;
    niveau              =  haut ;
    } // sortie : l=l, n=l+1.
courbe_carree [0] =  n - 2 ;
courbe_carree [n] =  l - 1 ;
return courbe_carree ;
}

/*----------------------------------------------------------------------
   ----------------------------------------------------------------------
    AlSeuilleCoupe (Seuille coupe)
     Prend une courbe de taille "l", la seuille avec le seuil donn et renvoit
   le signal en crneaux correspondant (Cf. spcifications dans LTATOU.H).
   */
BOOL AlSeuilleCoupe ( unsigned char *courbe
    , // courbe initiale  seuiller. Contenu inchang
      float seuil
    , int l // longueur de la courbe initiale (l<=256)
    , unsigned char *courbe_carree )
{
char  niveau = 0, bas = 0, haut = 1 ;
int   i = 0, n = 0 ;

//  courbe_carree doit etre alloue avec l+2 lments ou 3 si l==0.
if ( l < 0 || courbe_carree == NULL || courbe == NULL ) 
  return MErreur("Courbe de longueur ngative, par Cthulhu!", 2385);
else if ( l == 0 ) 
  {
  courbe_carree [0] =  0 ;
  courbe_carree [1] =  - 1 ;
  courbe_carree [2] =  0 ;
  return SUCCES ;
  }
for ( i = 0 ; ((float)(courbe [i]) == seuil) && (i < l) ; i++ ) 
  ;

/* Chercher le premier lment diffrent du seuil (qui dtermine le
   niveau de dpart). */
if ( i == l ) /* Pas d'lments, courbe plate gale au seuil. */ 
                {
                courbe_carree [0] =  0 ;
                courbe_carree [1] =  0 ;
                courbe_carree [2] =  l - 1 ;
                return SUCCES ;
                }

// Allou avec l+2 lments, l>=1, l+2 >= 3
if ( (float)(courbe [i]) > seuil ) 
  niveau =  haut ;
else 
  niveau =  bas ;
courbe_carree [1] =  niveau ;
n                 =  2 ;
for ( i = 1 ; i < l ; i++ ) 
  {
  if ( n >= l + 2 ) 
    return ErreurCode(704);
  if ( (niveau == haut) && ((float)(courbe [i]) < seuil) ) 
    {
    courbe_carree [n++] =  i - 1 ;
    niveau              =  bas ;
    }
  else if ( (niveau == bas) && ((float)(courbe [i]) > seuil) ) 
    {
    courbe_carree [n++] =  i - 1 ;
    niveau              =  haut ;
    } // "n++" n'est pas effectu  chaque boucle.
  }
courbe_carree [0] =  n - 2 ;
if ( n >= l + 2 ) 
  return ErreurCode(704);
courbe_carree [n] =  l - 1 ;
return SUCCES ;
}

/*----------------------------------------------------------------------
    AnalFreqCoupe (Analyse de frquence d'une coupe)
     Prend un signal en crneaux (voir spcifications dans LTATOU.H) et
   analyse sa frquence par limination rcursive des crneaux trop petits.
     La frquence rsultante est impaire car issue d'un symbole DM.
     La fonction reoit le signal et renvoit la frquence (int).
     La fonction alloue elle-mme et renvoit le tableau "histocases". L'ancien
   pointage d'"histocases" est perdu, il est vivement conseiller de s'assurer
   que la cellule dpointe ne se perde pas dans le nant.
   
     nf :  nombre de fronts,
     lt:           longueur totale du signal
     l :           longueur du signal sans les premire et dernire cases,
     pl :          pointeur sur l,
     pdeb :        pointeur sur l'adresse du premier lment (sans la premire
   case),
     cases :       tableau des longueur de case,
     histocases :  tableau histogramme des longueur de case,
     Phistocases : poiteur sur "histocases" pour tude de validit,
     petite_case, grande_case :    extrema de taille de case,
     case_maj :                    case la plus reprsente,
     gr_cases :                    cardinal d'un groupe de cases,
     f :                           frquence,
     nb_freq :             nombre de frquences reprsentes,
     nb_fus et nb_sep :    nombre de fusions et sparations,
     qpfm :                quantit de cases de la premire freq. maj.,
     no_fmaj :             numro de la frquence majoritaire,
     P... :                pointeur sur la variable"...".
   */
int AnalFreqCoupe ( unsigned char *creneaux, unsigned char **Phistocases
    , int *Pnb_freq, int *Pnb_fus, int *Pnb_sep, float *Pect_cases
    , int *Pqpfm )
{
int           nf = 0, i = 0, freq_min = 0, freq_max = 0, nb_freq = 0
  , freq_maj = 0, l = 0, dep_sgnl = 0, fin_sgnl = 0, no_fmaj = 0
  , verifindex = 0, f = 0 ;
int           nb_cases = 0, nb_cases_ap = 0 ;
unsigned char *cases = NULL ;
unsigned char *histocases = NULL ;
unsigned char petite_case = 0, grande_case = 0 ;
statistiques  stat_c = { 0, 0, 0, 0.0, 0.0, 0.0 };
char          prem_pass = VRAI ;

if ( creneaux == NULL || (nf = creneaux [0]) <= 3 ) 
  {
  histocases     =  (unsigned char *)malloc(1);
  histocases [0] =  0 ;
  *Phistocases   =  histocases ;
  *Pect_cases    =  *Pnb_freq = *Pnb_fus = *Pnb_sep = *Pqpfm = 0 ;
  return 0 ;
  }
l     =  creneaux [nf + 2] + 1 ;
cases =  (unsigned char *)malloc((nf + 1) * sizeof(unsigned char));
if ( cases == NULL ) 
  {
  Erreur("Erreur d'allocation (ref. 2999), blocage imminent");
  return 0 ;
  }
if ( creneaux [1] == 1 ) /* Premier niveau en haut (blanc). Ceci est un rsidu de la pente en ndg qui
                             mne au premier crneau et qui n'est pas assez raide. Des lments sont dans
                             le signal coup juste mais pas encore dans le niveau bas du premier crneau.*/ 
                          {
                          dep_sgnl  =  1 ; /* Le premier crneaux est le suivant. */ 
                          cases [0] =  creneaux [3] - creneaux [2];
                          }
else 
  cases [0] =  creneaux [2] + 1 ;
if ( (nf - dep_sgnl) % 2 ) 
  
  /* vrai si nf impair. Le front final a le mme dfaut
     que le front initial du cas prcdent.*/
  fin_sgnl =  - 1 ;
*Pnb_sep    =  0 ;
*Pnb_fus    =  0 ;
nb_cases_ap =  nf - dep_sgnl + fin_sgnl + 1 ;
if ( nb_cases_ap <= 3 ) 
  {
  free(cases);
  cases        =  NULL ; // Affectation anti-alas.
  histocases   =  (unsigned char *)malloc(1);
  *Phistocases =  histocases ;
  *Pect_cases  =  *Pnb_freq = *Pnb_fus = *Pnb_sep = *Pqpfm = 0 ;
  return 0 ;
  }
for ( i = 1 ; i < nb_cases_ap ; i++ ) 
  cases [i] =  creneaux [dep_sgnl + i + 2] - creneaux [dep_sgnl + i + 1];
stat_c =  Statistiques_courbe(cases, nb_cases_ap);
l      =  (int)stat_c.som ; /* Recalcul d'"l". */ 
do 
  {
  nb_cases    =  nb_cases_ap ;
  stat_c      =  Statistiques_courbe(cases, nb_cases);
  petite_case =  stat_c.min ;
  grande_case =  stat_c.max ;
  if ( !grande_case || !petite_case ) 
    return MErreur("Il manque une case (blocage imminent).", 761);
  freq_min =  PlusProcheImpair((float)l / (float)grande_case);
  freq_max =  PlusProcheImpair((float)l / (float)petite_case);
  nb_freq  =  (freq_max - freq_min) / 2 + 1 ;
  if ( freq_min <= 0 || freq_max <= 0 || nb_freq < 1 ) 
  return MErreur(
    "Rendez-vous, solaires, la flotte de Trantor est a votre porte (blocage imminent)."
    , 2538);
  if ( !prem_pass ) 
    {
    free(histocases);
    histocases =  NULL ;
    } // Affectation anti-alas.
  histocases =  (unsigned char *)malloc(nb_freq * sizeof(unsigned char));
  if ( histocases == NULL ) 
    return Erreur("Blocage imminent. Erreur d'allocation (ref. 3034)");
  for ( i = 0 ; i < nb_freq ; i++ ) 
    
    /* RAZ d'"histocases". */
    histocases [i] =  0 ;
  for ( i = 0 ; i < nb_cases ; i++ ) /* Remplissage d'"histocases". */ 
                                      {
                                      if ( !cases [i] ) 
  return Erreur("Blocage imminent, il manque une case pour i=", i, 776);
                                      f          =  PlusProcheImpair(
                                        (float)l / (float)cases [i]);
                                      verifindex 
                                        =  (f - (unsigned char)freq_min)
                                          / 2 ;
                                      if ( verifindex < 0
                                            || verifindex >= nb_freq ) 
  return MErreur(
    "Blocage imminent. Disparition de Dieu. Suite du programme impossible."
    , 2550);
                                      (histocases [verifindex])++ ;
                                      }
  no_fmaj =  (int)Loc_maximum_courbe(histocases, nb_freq);
  if ( no_fmaj < 0 || no_fmaj >= nb_freq ) 
  return MErreur(
    "Blocage imminent. Un tyranosaure a mang un circuit. Suite du programme impossible."
    , 2557);
  freq_maj =  freq_min + 2 * no_fmaj ; /* impair */ 
  if ( prem_pass ) 
    *Pqpfm =  (int)histocases [no_fmaj];
  if ( freq_maj == l + 1 ) 
    freq_maj -= 2 ; /* Cas o la priode majoritaire vaut un pixel et l pair. */ 
  if ( freq_maj != 1 && nb_cases_ap >= 3 ) 
    
    /* cas terminal. */
    pontage_long(cases, freq_maj, &nb_cases_ap);
  (*Pnb_fus)++ ;
  prem_pass =  FAUX ;
  } 
while ( nb_cases_ap < nb_cases );
(*Pnb_fus)-- ;

/* Plus de rduction de cases donc plus de rupture de seq. dans le
   signal. */
/* Etude des cas de "merge", c'est  dire de cases faces. */
do 
  {
  nb_cases    =  nb_cases_ap ;
  stat_c      =  Statistiques_courbe(cases, nb_cases);
  petite_case =  stat_c.min ;
  grande_case =  stat_c.max ;
  if ( !grande_case || !petite_case ) 
    return MErreur("Blocage imminent. Il manque encore une case.", 761);
  freq_min =  PlusProcheImpair((float)l / (float)grande_case);
  freq_max =  PlusProcheImpair((float)l / (float)petite_case);
  nb_freq  =  (freq_max - freq_min) / 2 + 1 ;
  if ( freq_min <= 0 || freq_max <= 0 || nb_freq < 1 ) 
  return MErreur(
    "Blocage imminent. Les filaments se sont chaps des ampoules du plafond."
    , 2583);
  free(histocases);
  histocases =  NULL ; // Affectation anti-alas.
  histocases =  (unsigned char *)malloc(nb_freq * sizeof(unsigned char));
  if ( histocases == NULL ) 
    return MErreur("Blocage imminent. Erreur d'allocation (ref. 3069)");
  for ( i = 0 ; i < nb_freq ; i++ ) 
    
    /* RAZ d'"histocases". */
    histocases [i] =  0 ;
  for ( i = 0 ; i < nb_cases ; i++ ) /* Remplissage d'"histocases". */ 
                                      {
                                      if ( !cases [i] ) 
  return Erreur("Blocage imminent.Il manque encore une case pour i=", i
    , 776);
                                      f          =  PlusProcheImpair(
                                        (float)l / (float)cases [i]);
                                      verifindex =  (f - freq_min) / 2 ;
                                      if ( verifindex < 0
                                            || verifindex >= nb_freq ) 
  return MErreur(
    "Blocage imminent. Interfrences du code avec des moucherons provenant de Mars. Programme interrompu."
    , 2593);
                                      (histocases [verifindex])++ ;
                                      }
  freq_maj =  freq_min + 2 * (int)Loc_maximum_courbe(histocases, nb_freq);
  if ( freq_maj == l + 1 ) 
    freq_maj -= 2 ; /* Cas o la priode majoritaire vaut un pixel et l pair. */ 
  if ( freq_maj != 1 && nb_cases_ap >= 3 ) 
    
    /* cas terminal. */
    hachage_long(&cases, freq_maj, &nb_cases_ap);
  (*Pnb_sep) += (nb_cases_ap - nb_cases) / 2 ;
  } 
while ( nb_cases_ap > nb_cases && *cases );

// S'arrete si cases est nul.
/* Plus de multiplication de cases donc plus de "merge" dans le
   signal. */
*Phistocases =  histocases ;
*Pnb_freq    =  nb_freq ;
*Pect_cases  =  stat_c.ecart_type ;
free(cases);
return nb_cases_ap ;
}

/*----------------------------------------------------------------------
    Pontage_long (Pontage longueur)
     Elimine dans une suites de crneaux (dfinis par leurs longueurs) ceux
   trop petits (fusionne les deux voisins). Le signal source est modifi
   directement ainsi que le nombre de crneaux pass par pointeur.
     Les crneaux trop petit sont ceux de taille infrieure au tiers de la
   priode  respecter. Ils sont fusionns si la taille finale n'atteit pas
   le double de cette priode. Ces deux paramtres 1/3 et 2 sont  rgler dans
   pontage_long.
     creneaux :    le signal source  modifier,
     freq :        frquence  respecter dans le signal,
     nf, nc :      respectivement le nombre de fronts et le nombre de crneaux,
     Pnc :         pointeur sur nc.
   */
void pontage_long ( unsigned char *creneaux, int freq, int *Pnc )
{
#define PERIODE_FUSION 1.0/3.0
#define LIMITE_FUSION 2.0

int           nc, l ; /* Nombre de crneaux et longueur du signal. */ 
int           i, j ;
unsigned char taille, taille_prec, taille_suiv, taille_min, taille_max, tc ;

/* Tailles des crneaux courantes, prec. et suiv. */
float         periode_inf, periode_sup, periode_maj ;
statistiques  stat_c ;

nc =  *Pnc ;
if ( nc < 3 ) 
  {
  MErreur(
    "Diminution inquitante de la gravit, veuillez m'attacher au bureau. Blocage imminent."
    , 2756);
  return ;
  }
if ( !(freq % 2) ) 
  {
  MErreur("Invasion de moucherons de Mars. Blocage imminent.", 2760);
  return ;
  }
if ( freq < 3 ) 
  {
  MErreur("Perte de protons inquitante. Blocage imminent.", 2762);
  return ;
  }
if ( creneaux == NULL ) 
  {
  MErreur(
    "Attaque par un lapin froce, intgrit du systme en pril. Blocage imminent."
    , 2764);
  return ;
  }
stat_c      =  Statistiques_courbe(creneaux, (unsigned char)nc);
l           =  (int)stat_c.som ;
periode_maj =  (float)l / (float)freq ;
periode_inf =  PERIODE_FUSION * periode_maj ;
periode_sup =  LIMITE_FUSION * periode_maj ;
taille_min  =  stat_c.min ;
taille_max  =  stat_c.max ;
for ( i = 0 ; creneaux [i] != taille_min ; i++ ) 
  ;

/* i pointe sur le premier crneau de taille minimale. */
taille =  taille_min ;
while ( taille < periode_inf ) 
  {
  if ( i == 0 ) 
    {
    taille_prec =  0 ;
    taille_suiv =  creneaux [1];
    }
  else if ( i == nc - 1 ) 
    {
    taille_suiv =  0 ;
    taille_prec =  creneaux [nc - 2];
    }
  else 
    {
    taille_prec =  creneaux [i - 1];
    taille_suiv =  creneaux [i + 1];
    }
  if ( taille_prec + taille + taille_suiv < periode_sup && i != 0 ) /* Pontage interdit en dbut de crneaux. */ 
                                                                      {
  nc -= 2 ;                                                           
  if ( i < 1 ) 
    {
    MErreur("Blocage imminent. Orange alert... Orange alert...", 2801);
    return ;
    }                                                                 
  creneaux [i - 1] =  taille_prec + taille + taille_suiv ;            
  for ( j = i ; j < nc ; j++ ) 
    
    /* Avalement du crneau. */
    creneaux [j] =  creneaux [j + 2];
                                                                      }
  taille_min =  taille ;
  taille     =  0 ;
  for ( j = i + 1 ; j < nc ; j++ ) 
    
    /* Chercher dans la suite un crneau de mme
       taille... */
    if ( creneaux [j] == taille_min ) 
      {
      taille =  taille_min ;
      break ;
      } /* ... ce crneaux existe. */ 
  if ( taille == 0 ) /* Dans l'autre cas : prendre les crneaux de taille
                         suprieure. */ 
                      {
                      taille =  taille_max + 1 ;
                      for ( i = 0 ; i < nc ; i++ ) 
                        {
                        tc =  creneaux [i];
                        if ( tc > taille_min ) 
                          if ( tc < taille ) 
                            {
                            taille =  tc ;
                            j      =  i ;
                            }
                        }
                      }
  i =  j ;
  }
*Pnc =  nc ;
#undef PERIODE_FUSION
#undef LIMITE_FUSION
}

/*----------------------------------------------------------------------*/
/* Hachage_long (Hachage longueur)
     Spare dans une suites de crneaux (dfinis par leurs longueurs) ceux
   trop grands (les coupe en 3 ou 5...). Le signal source est modifi
   directement ainsi que le nombre de crneaux pass par pointeur. Il sera
   r-allou si sa taille augmente (presque toujours le cas).
     Les crneaux trop grand sont ceux de taille suprieure au triple de la
   priode  respecter. Ce paramtre 3 est  rgler.
     creneaux :    le signal source  modifier,
     freq :        frquence  respecter dans le signal,
     nf, nc :      respectivement le nombre de fronts et le nombre de crneaux,
     Pnc :         pointeur sur nc.
   */
void hachage_long ( unsigned char **Pcreneaux, int freq, int *Pnc )
{
#define LIMITE_SEPARATION 2.0

int           nc = *Pnc, l = 0 ; /* Nombre de crneaux et longueur du signal. */ 
int           i = 0, j = 0 ;
unsigned char taille = 0, taille_min = 0, taille_max = 0, tc = 0
  , nb_sep = 0, *creneaux_av = NULL, *creneaux_ap = NULL ;

/* Tailles des crneaux courantes, prec. et suiv. */
float         periode_maj = 0.0, rap_t = 0.0 ;
statistiques  stat_c = { 0, 0, 0, 0.0, 0.0, 0.0 };

if ( *Pcreneaux == NULL ) 
  {
  MErreur(
    "Blocage imminent. Calvin voudrait savoir si vous n'avez pas vu Hobbes. Allez l'aider  le chercher."
    , 2848);
  return ;
  }
creneaux_av =  *Pcreneaux ;
creneaux_ap =  (unsigned char *)malloc(1 * sizeof(unsigned char));
if ( creneaux_ap == NULL ) 
  {
  MErreur("Blocage imminent. Erreur d'allocation (ref. 3452)");
  return ;
  }
if ( (nc < 3) || !(freq % 2) || (freq < 3) ) 
  {
  MErreur(
    "Blocage imminent. 3 lectrons sont manquants. Poursuite impossible."
    , 2853);
  return ;
  }
if ( (nc < 3) || !(freq % 2) || (freq < 3) ) 
  {
  MErreur(
    "Blocage imminent. Neutrinos emmls dans le cble de l'imprimmante."
    , 2863);
  return ;
  }
if ( (nc < 3) || !(freq % 2) || (freq < 3) ) 
  {
  MErreur(
    "Blocage imminent. Chute de la pression galactique. Poursuite impossible."
    , 2865);
  return ;
  }
stat_c =  Statistiques_courbe(creneaux_av, nc);
l      =  (int)stat_c.som ;
if ( l == 0 ) 
  {
  MErreur("Blocage imminent. Panne d'essence !", 2857);
  return ;
  }
periode_maj =  (float)l / (float)freq ;
taille_min  =  stat_c.min ;
taille_max  =  stat_c.max ;
for ( i = 0 ; creneaux_av [i] != taille_max ; i++ ) 
  ;

/* i pointe sur le premier crneau de taille maximale. */
taille =  taille_max ;
rap_t  =  (float)taille / periode_maj ;
while ( rap_t > LIMITE_SEPARATION ) 
  {
  nb_sep =  PlusProcheImpair(rap_t);
  nc     =  nc + nb_sep - 1 ;
  
  /* Modif de la liste des cren. */
  free(creneaux_ap); /* Cration du nouveau signal. */ 
  creneaux_ap =  NULL ; // Affectation anti-alas.
  creneaux_ap =  (unsigned char *)malloc(nc * sizeof(unsigned char));
  if ( creneaux_ap == NULL ) 
    {
    Erreur("Blocage imminent. Erreur d'allocation (ref. 3473)");
    return ;
    }
  for ( j = 0 ; j < i ; j++ ) 
    creneaux_ap [j] =  creneaux_av [j];
  for ( j = 0 ; j < nb_sep ; j++ ) 
    creneaux_ap [i + j]
      =  NUpRoundR((float)creneaux_av [i] / (float)nb_sep);
  for ( j = i + nb_sep ; j < nc ; j++ ) 
    creneaux_ap [j] =  creneaux_av [j - nb_sep + 1];
  taille_max =  taille ;
  taille     =  0 ;
  for ( j = i + 1 ; j < nc ; j++ ) 
    
    /* Chercher dans la suite un crneau de mme
       taille... */
    if ( creneaux_ap [j] == taille_max ) 
      {
      taille =  taille_max ;
      break ;
      } /* ... ce crneaux existe. */ 
  if ( taille == 0 ) /* Dans l'autre cas : prendre les crneaux de taille
                         infrieure. */ 
                      {
                      taille =  taille_min - 1 ;
                      for ( i = 0 ; i < nc ; i++ ) 
                        {
                        tc =  creneaux_ap [i];
                        if ( tc < taille_max ) 
                          if ( tc > taille ) 
                            {
                            taille =  tc ;
                            j      =  i ;
                            }
                        }
                      }
  i     =  j ;
  rap_t =  (float)taille / periode_maj ;
  free(creneaux_av); /* Reconstitution du signal  renvoyer. */ 
  creneaux_av =  (unsigned char *)malloc(nc * sizeof(unsigned char));
  if ( creneaux_av == NULL ) 
    {
    Erreur("Blocage imminent. Erreur d'allocation (ref. 3507)");
    return ;
    }
  for ( j = 0 ; j < nc ; j++ ) 
    creneaux_av [j] =  creneaux_ap [j];
  }
*Pnc       =  nc ;
*Pcreneaux =  creneaux_av ;
free(creneaux_ap);
creneaux_ap =  NULL ; // Affectation anti-alas.
#undef LIMITE_SEPARATION
}

/*----------------------------------------------------------------------
   -------------------------------------------------------------------------
    Critere_frequence (Critre de frquence)
     Calcule le critre selon la fonction dfinie (Cf. notes du 7 avril 1995).
     frequence :   frquence du signal reconstitu (et nombre de cases),
     qpfm :        quantit. de cases de la premire frquence majoritaire
                   calcule (avant la sparation_fusion),
     periode :     nombre de pixels de la priode corrige (en flottant, calcul
    partir de la frquence et du nombre de pixels total),
     ect_per :     cart type des tailles de priode,
     nb_fus :      nombre de fusions,
     nb_sep :      nombre de sparations,
     pof :         part d'origine de la frquence,
     etn :         cart type normalis (de taille de par.),
     nfs :         nombre de fusion/sparation,
     Cabs, Crel :  critre absolu et relatif.
   */
float Critere_frequence ( int frequence, int qpfm, float periode
    , float ect_per, int nb_fus, int nb_sep )
{
float Cabs, Crel, pof, nfs, etn ;

if ( (periode < 1e-4) || (frequence % 2 - 1) ) 
  return 1.0 ;
pof =  (float)qpfm / frequence ;
nfs =  (nb_fus + nb_sep) / (float)frequence ;
etn =  ect_per / periode - VN_etn ;
  {
  Cabs 
    =  exp( - K_pof * (pof - VN_pof)) + exp( + K_etn * (etn - VN_etn))
      + exp( + K_nfs * (nfs - VN_nfs));
  Crel =  atan(Cabs) / M_PI * 2.0 ;
  return Crel ;
  }
}

/*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
void DecCirc ( TQuadrilatere *pABCD
    , unsigned int *pwSymbolReadingProperties )
{
pABCD -> DecCirc();
if ( *pwSymbolReadingProperties & SMB_PROP_Origin ) 
  {
  
  // Origine est connue
  unsigned char cOrig
    = SMB_PROP_OriginA_D2Id(
        *pwSymbolReadingProperties & SMB_PROP_OriginA_D)
      - 'A';
  cOrig                      =  (cOrig + 3) % 4 ; // Dcalage inverse
  *pwSymbolReadingProperties &= ~SMB_PROP_OriginA_D ; // RAZ
  *pwSymbolReadingProperties |= SMB_PROP_Id2OriginA_D(cOrig + 'A');
  }
}

void ReOrgQuadMaxMax ( TQuadrilatere *pABCD
    , unsigned int *pwSymbolReadingProperties )
{
int iTrans = 0 ;

pABCD -> ReOrgQuadMaxMax(&iTrans);

/* AB est le plus grand des 2 cts, BC est le 2nd plus grand.
   La premire analyse est faites selon des tranches gnres sur [AB]
     est pseudo-paralleles  [AD] et [BC]. */
if ( iTrans < 0 ) 
  if ( *pwSymbolReadingProperties & SMB_PROP_Clock ) /* Le sens est connu et le symbole a t flipp */ 
                                                      {
  if ( *pwSymbolReadingProperties & SMB_PROP_ClockWise ) 
    *pwSymbolReadingProperties &= ~SMB_PROP_ClockWise ;
  else 
    *pwSymbolReadingProperties |= SMB_PROP_ClockWise ;
                                                      }
if ( *pwSymbolReadingProperties & SMB_PROP_Origin ) 
  {
  
  // Origine est connue
  switch ( *pwSymbolReadingProperties & SMB_PROP_OriginA_D ) 
    {
    case SMB_PROP_OriginA : 
      *pwSymbolReadingProperties &= ~SMB_PROP_OriginA_D ; // RAZ
      *pwSymbolReadingProperties |= SMB_PROP_OriginC ;
      break ;
    case SMB_PROP_OriginB : 
      *pwSymbolReadingProperties &= ~SMB_PROP_OriginA_D ; // RAZ
      *pwSymbolReadingProperties |= SMB_PROP_OriginD ;
      break ;
    case SMB_PROP_OriginC : 
      *pwSymbolReadingProperties &= ~SMB_PROP_OriginA_D ; // RAZ
      *pwSymbolReadingProperties |= SMB_PROP_OriginA ;
      break ;
    case SMB_PROP_OriginD : 
      *pwSymbolReadingProperties &= ~SMB_PROP_OriginA_D ; // RAZ
      *pwSymbolReadingProperties |= SMB_PROP_OriginB ;
      break ;
    }
  }
if ( *pwSymbolReadingProperties & SMB_PROP_Origin ) 
  {
  
  // Origine est connue
  unsigned char cOrig
    = SMB_PROP_OriginA_D2Id(
        *pwSymbolReadingProperties & SMB_PROP_OriginA_D)
      - 'A';
  cOrig                      =  (cOrig + 4 - abs(iTrans)) % 4 ; // Dcalage inverse
  *pwSymbolReadingProperties &= ~SMB_PROP_OriginA_D ; // RAZ
  *pwSymbolReadingProperties |= SMB_PROP_Id2OriginA_D(cOrig + 'A');
  }
}

BOOL AnalBiFreqQuadGeneriquePM ( TQuadrilatere ABCD // Variable locale
, unsigned int *pwFreq, unsigned char *puSource
    , unsigned int wSymbolReadingProperties // Variable locale
    , unsigned char uMode
    , unsigned char *puTabFreqSvg, unsigned char *puTabCritSvg
    , unsigned int *pwNbCoupes, float *axCellTableC1, float *axCellTableC2
    , TBipoint *pbipCellBearerC1, TBipoint *pbipCellBearerC2 )
{
unsigned int  wF2 ;

/*
   
   La quadrilatre pass est suppos avoir le plus grand voisin du ct en tte en second.
   
     On va dcider que: si une table des cellules est demand, et si
     l'analyse de la frquence est positive, on fait une 2e analyse
     avec le quadrilatre tel que le nouveau ct en tte est le plus
     grand parmis les 2 ct voisins de l'ancien AB.
   
      Si l'analyse de la frquence est bonne aussi et si la frquence est la
    mme, alors on propage vers le haut les deux grilles+la direction  laquelle   elles
    sont associ.
      Au processi postrieurs de retrouver la grille 2D  partir des 2 grilles
    1D (compltion avec les points de fuite).
      Il faut donc remplacer transformer le quadrilatre ABCD en un
    quadrilatre tel que:
    AB est le plus grand des cts et BC le plus grand de BC et AD.
   */
ReOrgQuadMaxMax(&ABCD, &wSymbolReadingProperties);
if ( pwFreq
      && AnalFreqQuadGenerique(ABCD, pwFreq, puSource
        , wSymbolReadingProperties, uMode, puTabFreqSvg, puTabCritSvg
        , pwNbCoupes, axCellTableC1, pbipCellBearerC1) ) 
  {
  
  /* On fait l'analyse 2 fois pour reseigner les tables de cellules et pour
     s'assurer que la freq. est identique dans les 2 directions.*/
  DecCirc(&ABCD, &wSymbolReadingProperties); // Cette rorganisation n'est pas rmanente
  
  /* AB est le plus grand des 2 cts, BC est le 2nd plus grand.
     La seconde analyse est faite selon des tranches gnres sur [BC]
     est pseudo-paralleles  [BA] et [CD].
     
     
     */
  return AnalFreqQuadGenerique(ABCD, &wF2, puSource
            , wSymbolReadingProperties, uMode, puTabFreqSvg, puTabCritSvg
            , pwNbCoupes, axCellTableC2, pbipCellBearerC2)
          && wF2 == *pwFreq ;
  }
else 
  return ECHEC ;
}

BOOL AnalFreqQuadGenerique ( TQuadrilatere ABCD, unsigned int *pwFreq
    , unsigned char *puSource, unsigned int wSymbolReadingProperties
    , unsigned char uMode, unsigned char *puTabFreqSvg
    , unsigned char *puTabCritSvg, unsigned int *pwNbCoupes
    , float *axCellTable, TBipoint *pbipCellBearer )
{
/*
   modes A : application et D : dmonstration.
   QUAD_APPLICATION et QUAD_INSPECTION.
   
   La procdure reoit un quadrilatre ABCD o AMHA [AB] n'est pas
   ncessairement le plus grand de tous les cts, mais ncessairement
   plus grand que son vis--vis.
   */
char          fBalayage = BALAYAGE_EN_COLONNE, cSgnDeltaLigne = 0
  , cSgnDeltaColonne = 0, cSgnDelta = 0 ;
int           iDeltaLigne = 0, iDeltaColonne = 0, iDelta = 0 ;
unsigned int  wNbCoupes ;
unsigned char *puTabFreq = NULL ;
unsigned char *puTabCrit = NULL ;

try
  {
  if ( uMode != QUAD_APPLICATION
        && uMode != QUAD_INSPECTION
        && uMode != ANLFREQ_LocaliseTranche ) 
    {
    throw MErreur("Mode inconnu.", 1334);
    ;
    ;
    }
  if ( uMode == QUAD_INSPECTION ) 
    {
    if ( !pwNbCoupes || !puTabFreqSvg || !puTabCritSvg ) 
      {
      
      // Mode QUAD_INSPECTION
      throw MErreur("Pointeurs nuls.", 684);
      ;
      ;
      }
    }
  } catch ( int ihExc ) 
  {
  return ihExc ;
  }
/*
       [AB] n'est pas ncessairement le plus grand de tous les cts, mais
     ncessairement plus grand que son vis--vis, ce qui permet que le
     gnrateur de tranches de l'tape suivante donne des tranches jointives.
   
   
     Numrise: une ligne ressemble :
   
   aXXXXXXXib
            cXXXXXXXjd
                     eXXXXXXXkf
                              gXXlh
   
     Pour la balayer, ou peut faire de diffrentes faons.
       1: Tous les pixels:
         a-b puis c-d puis e-f puis g-h
         C'est le balayage que l'on suit lorsque l'on colorie la ligne sur un cran.
       2: Tous les pixels sauf ceux des nuds
         a-i puis c-j puis e-k, puis g-h
         Ce sont les pixels que l'on voit sui on regarde la ligne depuis l'axe horizontal
       3: Uniquement ceux des nuds
         a, c, e, g
         Ce sont les pixels que l'on voit sui on regarde la ligne depuis l'axe vertical
   
     Nous avons adopt le balayage 2.
     Selon que la ligne est plutt de faible ou de forte pente, ce balayage 2 correspondra
     aux pixels vus depuis l'axe des 'x' ou des 'y'.
   
   
      */
iDeltaLigne      =  ABCD.B.x - ABCD.A.x ;
cSgnDeltaLigne   =  signe(iDeltaLigne);
iDeltaColonne    =  ABCD.B.y - ABCD.A.y ;
cSgnDeltaColonne =  signe(iDeltaColonne);
if ( abs(iDeltaLigne) > abs(iDeltaColonne) ) 
  {
  
  // Forte pente
  fBalayage =  BALAYAGE_EN_LIGNE ;
  iDelta    =  iDeltaLigne ;
  cSgnDelta =  cSgnDeltaLigne ; //Les pixels balays sont les intersections des lignes de l'image avec [AB]
  }
else 
  {
  
  // Faible pente
  fBalayage =  BALAYAGE_EN_COLONNE ;
  iDelta    =  iDeltaColonne ;
  cSgnDelta =  cSgnDeltaColonne ; //Les pixels balays sont les intersections des colonnes de l'image avec [AB]
  }
wNbCoupes =  abs(iDelta) + 1 ;
try
  {
  /*
     types d'erreurs: RAZ  effectuer:
     1 (0x01): RAZ *pwNbCoupes
     2 (0x02): RAZ *pwFreq
     3 (0x04): free puTabFreq
     4 (0x08): free puTabCrit
     */
  int ihMinError = 0x02 ; // free both+ RAZ *pwFreq
  if ( uMode == QUAD_INSPECTION && wNbCoupes > *pwNbCoupes ) 
    
    // Type erreur 1 : RAZ *pwNbCoupes, *pwFreq
    throw ihMinError | 0x01
      | Erreur("Taille des table de sauvegarde insuffisante NbCoupes="
        , (int)wNbCoupes, 1429);
  ;
  ;
  ;
  ;
  ;
  if ( uMode == ANLFREQ_LocaliseTranche ) 
    {
    CalculeCourbeFreqQuad(ABCD.A, ABCD.B, ABCD.C, ABCD.D, iDelta, cSgnDelta
      , fBalayage, wSymbolReadingProperties, NULL, NULL, NULL
      , ANLFREQ_LocaliseTranche, NULL, pbipCellBearer, *pwFreq);
    return SUCCESS ;
    }
  if ( (puTabFreq = (unsigned char *)malloc(wNbCoupes)) == NULL ) 
    throw ihMinError | 0x01
      | MErreur("Death has been allocated for you... !", 750);
  ;
  ;
  ;
  ;
  ;
  if ( (puTabCrit = (unsigned char *)malloc(wNbCoupes)) == NULL ) 
    
    // Type erreur 2 : RAZ *pwNbCoupes, *pwFreq, free puTabFreq
    throw ihMinError | 0x01 | 0x04
      | MErreur("A little place has been allocated for you in Hell. "
        , 1357);
  ;
  ;
  ;
  ;
  ;
  ihMinError |= 0x04 | 0x08 ; // free both+ RAZ *pwFreq
  /*
      puTabFreq et puTabCrit sont allous de taille wNbCoupes.
     Cette procdure calcule pour chaque coupe du quadrilatere la frquence
     et
     le critre.
     NB: BALAYAGE_EN_LIGNE=VRAI, forte pente pente (c-a-d W>H avec l'axe des 'x' vertical).
     Ce programme peut renvoyer axCellTable d'un critre minimal.
                                   ^^^^^^^^^^^^^^^^^^^^^^^^^^^
     */
  if ( !CalculeCourbeFreqQuad(ABCD.A, ABCD.B, ABCD.C, ABCD.D, iDelta
        , cSgnDelta, fBalayage, wSymbolReadingProperties, puTabFreq
        , puTabCrit, puSource, ANLFREQ_ToutesFrequences, axCellTable
        , pbipCellBearer) ) 
    
    // type erreur 3  RAZ *pwNbCoupes
    throw ihMinError | 0x01 | MErreur("Welcome in Bugland !", 1359);
  ;
  if ( uMode == QUAD_INSPECTION )
    {
    *pwNbCoupes =  wNbCoupes ;
    memcpy(puTabFreqSvg, puTabFreq, wNbCoupes);
    memcpy(puTabCritSvg, puTabCrit, wNbCoupes);
    }
  unsigned int  wRetries = 3 ;
  unsigned int  wIndexFreq = - 1 ;
  if ( !AnalyseCourbeFreqQuad(puTabFreq, puTabCrit, wNbCoupes, pwFreq
        , &wIndexFreq, &wRetries) )

    /*  Cette proc xamine la table des crit et frquences, et indique le
       meilleur des minima susceptible d'tre celui de la zone d'horloge.
         wRetries est le nombre de minima examins. */
    throw ihMinError
//                   | (0 & MTrace("Pas de frquence cohrente", 1461))
                   ;

  if ( !CalculeCourbeFreqQuad(ABCD.A, ABCD.B, ABCD.C, ABCD.D, iDelta
        , cSgnDelta, fBalayage, wSymbolReadingProperties, puTabFreq
        , puTabCrit, puSource, ANLFREQ_UneTranche, axCellTable
        , pbipCellBearer, wIndexFreq) ) 
    
    /* Cette proc va dterminer la table des cellules de la tranche indique par
       wIndexFreq que vient de calculer AnalyseCourbeFreqQuad.*/
    throw ihMinError
      | MErreur("Analyse de tranche spcifique impossible", 799);
  ;
  ;
  ;
  ;
  ;
  free(puTabCrit);
  puTabCrit =  NULL ; // Affectation anti-alas.
  free(puTabFreq);
  puTabFreq =  NULL ; // Affectation anti-alas.
  return SUCCES ;
  } catch ( int ihExec ) 
  {
  if ( ihExec & 0x01 ) 
    if ( uMode == QUAD_INSPECTION ) 
      *pwNbCoupes =  0 ;
  if ( ihExec & 0x02 ) 
    *pwFreq =  0 ;
  if ( ihExec & 0x04 ) 
    {
    free(puTabFreq);
    puTabFreq =  NULL ;
    } // Affectation anti-alas.
  if ( ihExec & 0x04 ) 
    {
    free(puTabCrit);
    puTabCrit =  NULL ;
    } // Affectation anti-alas.
  return FAILURE ;
  }
}

/*-------------------------------------------------------------------------*/
/*-------------------------------------------------------------------------
   
   // Coupe le quadrilatre en tranches et dtermine la frquence et le
   critre de chaque tranche.
   // Les tranches sont gnres par chaque pixel d'[AB] balay en ligne ou
   en colonne.
   // Normalement, le choix ligne/colonne assure que les pixels qui gnrent
   les tranches sont jointifs.
   */
BOOL CalculeCourbeFreqQuad ( TCoordonnee AN, TCoordonnee BN, TCoordonnee CN
    , TCoordonnee DN, int iNbElements, char cSensParcours, char fBalayage
    , unsigned int SymbolReadingProperties, unsigned char *auTableFreq
    , unsigned char *auTableCrit, unsigned char *puSource
    , unsigned char uMode, float *axCellTable, TBipoint *pbipCellBearer
    , unsigned int wTrancheAAnalyser )
{
/* auTableCrit et auTableFreq sont dj allous et de taille abs(iNbElements)+1.
Ils recevront la table des cellules du dernier critre minimal (la table
des cellules pour wTrancheAAnalyser dans le cas UNETRANCHE.
                                      // iNbElements peut tre ngatif.
*/

//  unsigned int SymbolReadingProperties=0;
int           k ;
TCoordonnee   ANi, DNi ;
TBipoint      bipLigne ;
TVecteur      A (AN), B (BN), C (CN), D (DN) ;
float         xCoefExtraction = 0.0 ;
int           l = 0, iFrequence = 0 ;
int           iAbscisseDepart = 0, iAbscisseArriveePlus1 = 0 ;
unsigned char auLigne [MAX_DIAGONAL]; // , *histocases = NULL ;
float         axBufferCellTable [MAX_DIAGONAL]
  , *axLocalCellTable = axCellTable ? axBufferCellTable : NULL ; // Is NULL or Buffer according if calculation has been asked or not.
unsigned int  wPeriodeMin = 0, wPeriodeMax = 255 ;
unsigned int  wLongueurLigne = 0 ;
statistiques  statLigne = { 0, 0, 0, 0.0, 0.0, 0.0 };
// int           iNbFrequences = 0,  iNbFusions = 0 , iNbSeparations = 0  ;
// int           iQPFM = 0 ;
float         xEcartTypePeriode = 0.0, xPeriode = 0.0 ;

//  MTrace("Entre CalculeCourbeFreqQuad",1540);
if ( !iNbElements ) 
  return MErreur("Danger Will Robinson (unexpected null value)", 1473);
if ( (uMode != ANLFREQ_LocaliseTranche)
      && (!auTableCrit || !auTableFreq || !Numeroplan(puSource)) ) 
  return MErreur("Danger Will Robinson (unexpected null pointer)", 1473);
/*
   fTravailEnLigne est vrai si le balayage se fait selon x,
   c'est a dire si la largeur est plus grande que la hauteur
   */
if ( fBalayage == BALAYAGE_EN_LIGNE ) 
  {
  iAbscisseDepart       =  AN.x ;
  iAbscisseArriveePlus1 =  BN.x + cSensParcours ;
  }
else 
  {
  
  // BALAYAGE_EN_COLONNE
  iAbscisseDepart       =  AN.y ;
  iAbscisseArriveePlus1 =  BN.y + cSensParcours ;
  }

float xMinimalCriterium = 2.0 ;

try
  {
  for ( l = iAbscisseDepart, k = 0 ; l != iAbscisseArriveePlus1 ; 
        l += cSensParcours, k++ ) 
    {
    if ( (uMode != ANLFREQ_UneTranche && uMode != ANLFREQ_LocaliseTranche)
          || k == wTrancheAAnalyser ) 
      {
      
      // xCoefExtraction >=0, car  iNbElements et cSensParcours sont de mme signe
      xCoefExtraction =  ((float)(l - iAbscisseDepart)) / iNbElements ; // De 0.0  ~1.0
      
      // Parcours de B vers A.
      ANi             =  TCoordonnee(
        TVecteur::Barycentre(A, xCoefExtraction, B, 1.0 - xCoefExtraction));
      DNi             =  TCoordonnee(
        TVecteur::Barycentre(D, xCoefExtraction, C, 1.0 - xCoefExtraction));
      bipLigne        =  TBipoint(ANi, DNi);
      if ( uMode != ANLFREQ_LocaliseTranche ) 
        {
        
        //    uLongueurLigne=max(abs(bipLigne.top-bipLigne.bottom),abs(bipLigne.right-bipLigne.left));
        MemoLigne(puSource, bipLigne, auLigne, &wLongueurLigne);
        statLigne =  Statistiques_courbe(auLigne, wLongueurLigne);
        if ( k > abs(iNbElements) ) 
          return ErreurCode(1560);
        }
      BOOL  fRacD, fRacF, fCompatibilite = 0 ;
      float xSeuilRaccourcissement ;
      switch ( uMode ) 
        {
        case ANLFREQ_FrequenceImpaire : 
          MErreur("Obsolete function");
/*
            {
            unsigned char *uLigneAuFormatCarre = NULL ;
            if ( (uLigneAuFormatCarre // Allocation
                = (unsigned char *)malloc(3 + (int)wLongueurLigne)) == NULL ) 
              return ErreurCode(1559);
            
            // Transformation du signal dans un format plus pratique (?)  utiliser (signal en crneaux)
            if ( !AlSeuilleCoupe(auLigne, statLigne.moy, wLongueurLigne
                  , uLigneAuFormatCarre) ) 
              {
              free(uLigneAuFormatCarre);
              throw ErreurCode(1691);
              ;
              }
            iFrequence
              =  AnalFreqCoupe(uLigneAuFormatCarre, &histocases
                , &iNbFrequences, &iNbFusions, &iNbSeparations
                , &xEcartTypePeriode, &iQPFM);
            
            // Histocases : allou par AnalFreqCoupe
            free(histocases);
            free(uLigneAuFormatCarre);
            uLigneAuFormatCarre =  NULL ;
            }
*/
          break ;
        case ANLFREQ_ToutesFrequences : 
        case ANLFREQ_UneTranche : 
          {
          unsigned int  LineReadingProperties = 0 ;
          SetBitWise(&LineReadingProperties, SMB_PROP_Origin
            , SymbolReadingProperties & SMB_PROP_Origin);
          if ( SymbolReadingProperties & SMB_PROP_Origin ) 
            {
            
            // On connat l'origine du symbole.
            if ( (SymbolReadingProperties & SMB_PROP_OriginA_D)
                    == SMB_PROP_OriginA
                  || (SymbolReadingProperties & SMB_PROP_OriginA_D)
                    == SMB_PROP_OriginB ) 
              LineReadingProperties |= SMB_PROP_OriginA ; // Motif en L en dbut de ligne
            else 
              LineReadingProperties |= SMB_PROP_OriginB ; // Motif en L en fin de ligne
            }
          
          // Symble est positif ou ngatif
          SetBitWise(&LineReadingProperties, SMB_PROP_Appearance
            , SymbolReadingProperties & SMB_PROP_Appearance);
          SetBitWise(&LineReadingProperties, SMB_PROP_AppPositive
            , SymbolReadingProperties & SMB_PROP_AppPositive);
          SetBitWise(&LineReadingProperties, SMB_PROP_Parity
            , SymbolReadingProperties & SMB_PROP_Parity);
          SetBitWise(&LineReadingProperties, SMB_PROP_Even
            , SymbolReadingProperties & SMB_PROP_Even);
          if ( !AnalyseFrequenceLigne(auLigne, wLongueurLigne
                , statLigne.moy // Le signal d'horloge est seuill selon la moyenne des luminances.
                , &iFrequence, &xPeriode, &xEcartTypePeriode
                , &wPeriodeMin, &wPeriodeMax, &fCompatibilite, &fRacD
                , &fRacF, &xSeuilRaccourcissement, LineReadingProperties
                , axLocalCellTable) ) 
            
            // auLigne modifie par AnalyseFrequenceLigne.
            //           ^^^^^^^^^^^^^^^^^
            throw MErreur("Erreur AnalyseFrequenceLigne", 1709);
          ;
          ;
          ;
          ;
          break ;
          }
        case ANLFREQ_LocaliseTranche : 
          *pbipCellBearer = bipLigne ;
          return SUCCESS ;
        default : 
          throw ErreurCode(1714);
          ;
          ;
          ;
          ;
        }
//      histocases =  NULL ; // Initialisation anti alas.
      if ( iFrequence < 0 ) 
        return MErreur("Godzilla attacks San-Francisco", 1477);
      else if ( iFrequence == 0 ) 
        xPeriode =  wLongueurLigne ;
      else if ( iFrequence > SIZE_OF_PICTURES - 1 ) 
        {
        Erreur("Saturation  haute frquence : f=", iFrequence, 1484);
        iFrequence =  SIZE_OF_PICTURES - 1 ;
        xPeriode   =  (float)wLongueurLigne / iFrequence ;
        }
/*
// Priode mesure sur la taille externe, sans tenir compte des rejets
// Ceci causerait des incohrences, comme par exemple xPeriode > wPeriodeMax (sic)


*/
      else
        xPeriode =  (float)wLongueurLigne / iFrequence ;

      if ( k < 0 || k >= abs(iNbElements) + 1 )
        return MErreur("index hors espace autoris", 1594);
      auTableFreq [k] =  (unsigned char)iFrequence ;
      float xCritere = 0.0 ;
      switch ( uMode )
        {
        case ANLFREQ_FrequenceImpaire :
// Obsolete
/*
          auTableCrit [k]
            = NUpRoundR(
              255.0
                * Critere_frequence(iFrequence, iQPFM, xPeriode
                  , xEcartTypePeriode, iNbFusions, iNbSeparations));
*/
          break ;
        case ANLFREQ_ToutesFrequences :
        case ANLFREQ_UneTranche :
          if ( fCompatibilite & ANLFREQ_MSG_RaccAveugle )

            /* La frquence a t ajuste aprs l'analyse (RaccD ou RaccF) de manire  coller
               avec la parite (LineReadingProperties) connue. Cet ajustement s'est fait sans
               info prcise sur la localisation des crneaux supprims. La localisation
               du porteur de cellule n'est donc pas significatif.
               */
            bipLigne =  bipErreur ;
          if ( fCompatibilite & ANLFREQ_MSG_NonCompatible )
            xCritere =  1.0 ;
          else if ( !CritereValiditeFrequenceDUneLigne(
                    iFrequence, xPeriode
                    , xEcartTypePeriode
                    , wPeriodeMin > 255 ? 255 : wPeriodeMin
                    , wPeriodeMax > 255 ? 255 : wPeriodeMax, &xCritere) ) 
            return MErreur("Erreur CritereValiditeFrequenceDUneLigne", 1748);
          auTableCrit [k] = NReduceR(255.0 * xCritere, 0.0, 255.0);
          break ;
        default : return ErreurCode(1752);
        }
      
      // Ici, on connat le critre. Lorsque celui-ci est minimal, on mmorise la table des cellules.
      if ( xCritere < xMinimalCriterium && axCellTable && pbipCellBearer ) 
        {
        xMinimalCriterium =  xCritere ;
        memcpy(axCellTable, axLocalCellTable, iFrequence * sizeof(float));
        
        /*      for (int i=0;i<iFrequence;i++)
           axCellTable[i]=axLocalCellTable[i]; */
        *pbipCellBearer =  bipLigne ;
        }
      }
    }
  } catch ( int ihExc ) 
  {
  return ihExc ;
  }

  /*CellTable connat la table des cellules de la
tranche au critre minimal  :) */

return SUCCES ;
}

/*-------------------------------------------------------------------------*/
BOOL AnalyseCourbeFreqQuadMinCritDeb ( unsigned char *auTabFreq
    , // Table des frquences
      unsigned char *auTabCrit // Table des critres
      ,
                                int iNbCoupes // Nb de coupes
    ,
      unsigned int *pwFreq// Frquence trouve (paramtre out)
      ,
                            unsigned char uIdxCritMin // Index du critre minimal (param in)
    ,
      unsigned int *pwIndexFreq
    ,
      unsigned int *pwRetries // Nombre de tentatives autorises restantes (param in out)
      )
{
if ( !auTabFreq || !auTabCrit || !pwFreq || !iNbCoupes )
  return MErreur("Analyse impossible", 1737);

unsigned int  wSd = 3 ;             // wSd: limite maximale d'loignement du dbut du puits par rapport au bord du symbole.
float         k1 = 0.15, d1 = 2.0 ; // d1 est mis  2, notament pour le cas o les 2 lignes de pixels qui bordent le zone d'horloge ont chou en lecture.

*pwFreq =  auTabFreq [uIdxCritMin]; // La frquence du min dsign

float         xCritMoyen = Moyenne_courbe(auTabCrit, iNbCoupes);  // Moy. des critres

// xCritMoyen=Critre moyen sur la courbe
unsigned int  wSm
  = NFarRoundR((auTabCrit [uIdxCritMin] + xCritMoyen) / 2.0);
/* Seuil de valeur de critre en de duquel les critres voisins du puits
lui sont associs.
*/
for ( unsigned int w = uIdxCritMin ;
      w + 1 != 0 /*equivallent to: w!=-1*/
        && auTabCrit [w] <= wSm
        && *pwFreq == auTabFreq [w] ;
      w-- )
  ;

unsigned int  w_id = w + 1 ;

//adresse du dernier point du puits qui prcde le min.
if ( w_id > wSd )
  return ECHEC ; // Le dbut du puits est trop loin du bord.
for ( w = uIdxCritMin ;
      w < iNbCoupes && auTabCrit [w] <= wSm && *pwFreq == auTabFreq [w] ;
      w++ )
  ;

unsigned int  w_ip = w - 1 ;           // Adresse du dernier point du puits qui suit le min.
unsigned int  w_t0 = w_ip - w_id + 1 ; // Largeur du puits
float         x_t0Min = MAXFLOAT, x_t0Max = MINFLOAT ;

if ( auTabFreq [uIdxCritMin] )
  {
  float x_PeriodeNominale = (float)iNbCoupes / auTabFreq [uIdxCritMin];

  // k1: marge dynamique (mult.) sur la priode
  // d1: marge statique (add.) sur la priode
  x_t0Min =  (1.0 - k1) * x_PeriodeNominale - d1 ;
  x_t0Max =  (1.0 + k1) * x_PeriodeNominale + d1 ;
  }
if ( w_t0 < x_t0Min || w_t0 > x_t0Max )
  {

  // La taille du puits ne correspond pas  la priode.
  if ( !pwRetries )

    // Pas de tentatives multiples autorises
    return ECHEC ;
  else if ( *pwRetries )
    {

    // Il reste des tentatives  essayer
    auTabCrit [uIdxCritMin] =  255 ; // Abandon de ce min.
    (*pwRetries)-- ; // Dcrmentation du nombre d'essais restants
    return AnalyseCourbeFreqQuad(auTabFreq, auTabCrit, iNbCoupes, pwFreq
      , pwIndexFreq, pwRetries); // Nouvelle tentative(s)
    }
  else
    return FAILURE ; // Plus de tentatives autorises.
  }
if ( pwIndexFreq )
  *pwIndexFreq =  uIdxCritMin ;
return SUCCES ;
}

/*-------------------------------------------------------------------------*/
// Locate the best frequence.
// i.e.: la frquence qui a le meilleur critre si un certain nombre de condition sont remplies.
char AnalyseCourbeFreqQuad ( unsigned char *auTabFreq
    , unsigned char *auTabCrit, int iNbCoupes, unsigned int *pwFreq
    , unsigned int *pwIndexFrequence, unsigned int *pwRetries )
{

//  Trace("Entre AnalyseCourbeFreqQuad, iNbCoupes=",iNbCoupes,1721);
if ( iNbCoupes > SIZE_OF_PICTURES ) 
  return Erreur("Dpassement du nombre d'chantillons, NbCoupes="
    , iNbCoupes, 918);
if ( iNbCoupes <= 0
      || auTabFreq == NULL
      || auTabCrit == NULL
      || pwFreq == NULL ) 
  return Erreur("Tableau non allou", iNbCoupes, 920);

unsigned char uIdxCritMin = Loc_minimum_courbe(auTabCrit, iNbCoupes);

// Procdure non vrifie lors du bug d'aout 1996.
if ( pwIndexFrequence ) 
  *pwIndexFrequence =  uIdxCritMin ;
if ( uIdxCritMin >= iNbCoupes ) 
  return Erreur("Echec  la recherche du minimum, uIdxCritMin="
    , uIdxCritMin, 1532);
if ( uIdxCritMin <= iNbCoupes / 2 ) 
  return AnalyseCourbeFreqQuadMinCritDeb(auTabFreq, auTabCrit, iNbCoupes
    , pwFreq, uIdxCritMin, pwIndexFrequence, pwRetries);
else 
  {
  // Duplication des tables, inversion du sens du signal afin de mettre le min. au dbut
  unsigned char *auNewTabFreq = (unsigned char *)malloc(iNbCoupes);
  unsigned char *auNewTabCrit = (unsigned char *)malloc(iNbCoupes);
  for ( int i = 0 ; i < iNbCoupes ; i++ )
    {
    auNewTabCrit [i] =  auTabCrit [iNbCoupes - i - 1];
    auNewTabFreq [i] =  auTabFreq [iNbCoupes - i - 1];
    }
  uIdxCritMin =  iNbCoupes - uIdxCritMin - 1 ;
  BOOL  f
    = AnalyseCourbeFreqQuadMinCritDeb(auNewTabFreq, auNewTabCrit
      , iNbCoupes, pwFreq, uIdxCritMin, pwIndexFrequence, pwRetries);
  if ( pwIndexFrequence )
    *pwIndexFrequence =  iNbCoupes - *pwIndexFrequence - 1 ; // On retourne l'index renvoy si les tables sont inverses.
  free(auNewTabCrit);
  auNewTabCrit =  NULL ; // Initialisation anti alas.
  free(auNewTabFreq);
  auNewTabFreq =  NULL ; // Initialisation anti alas.
  return f ;
  }
}

/*-------------------------------------------------------------------------*/
BOOL AnalyseFrequenceLigne ( unsigned char *auLigne
    , unsigned int wLongueurLigne, float xValeurSeuil, int *piFrequence
    , float *pxPeriode, float *pxEcartTypePeriode
    , unsigned int *pwPeriodeMin, unsigned int *pwPeriodeMax
    , BOOL *pfCompatibilite, BOOL *pRacourcissementD
    , BOOL *pRacourcissementF, float *pxSeuilRaccourcissement
    , unsigned int wProperties, float *axCellTable )
{
if ( wLongueurLigne ) 
  {
  unsigned char *puLigne = (unsigned char *)malloc(wLongueurLigne);
  memcpy(puLigne, auLigne, wLongueurLigne);
  
  // Fabrique une cop_ie de la ligne et lance l'analyse destructrice.
  BOOL  f
    = AnalyseDestFrequenceLigne(puLigne, wLongueurLigne, xValeurSeuil
      , piFrequence, pxPeriode, pxEcartTypePeriode, pwPeriodeMin
      , pwPeriodeMax, pRacourcissementD, pRacourcissementF
      , pxSeuilRaccourcissement, pfCompatibilite, wProperties, axCellTable);
  free(puLigne);
  return f ;
  }
else 
  return ECHEC ;
}

BOOL AnalyseDestFrequenceLigne ( unsigned char *auLigne
    , unsigned int wLongueurLigne, float xValeurSeuil, int *piFrequence
    , float *pxPeriode, float *pxEcartTypePeriode
    , unsigned int *pwPeriodeMin, unsigned int *pwPeriodeMax
    , BOOL *pRacourcissementD, BOOL *pRacourcissementF, float *pxSeuilRejet
    , BOOL *pfCompatibilite, unsigned int wProperties, float *axCellTable )
{
if ( !auLigne
      || !piFrequence
      || !pxPeriode
      || !pxEcartTypePeriode
      || !pwPeriodeMin
      || !pwPeriodeMin
      || !wLongueurLigne
      || wLongueurLigne == 1
      || xValeurSeuil < 0.0
      || !pRacourcissementD
      || !pRacourcissementF
      || !pxSeuilRejet ) 
  return ECHEC ;
*pfCompatibilite =  0 ;
if ( !SeuilleLigne(auLigne, wLongueurLigne, xValeurSeuil) )
  return ECHEC ;

// Il y a au plus "wLongueurLigne"-1 transitions et wLongueurLigne periodes.
/* Le nombre de priodes en fonction du nombre de contours dpend de la nature
   des extrmits.
     Dans le cas d'un symbole  frquence impaire, on aura en gnral un contour 
   chaque extrmit.
     Dans le cas d'un symbole  frquence paire, on aura en gnral un contour d'un
   ct uniquement.
     Pour l'instant, nous appelons periode la zone comprise entre 2 transitions ou
   entre le bord de la ligne et une transition. Cela fait une periode de plus que de transitions.
   */
//  axCellTable[wLongueurLigne]; // Table of center of cell.
//  Construisons un tableau des priodes.
unsigned int  *pwTablePeriodes
  = (unsigned int *)malloc(wLongueurLigne * sizeof(unsigned int));

if ( !pwTablePeriodes ) 
  return ECHEC ;

//    Recherche de la premire transition.
unsigned int  w ;

for ( w = 0 ; w < wLongueurLigne - 1 ; w++ ) 
  if ( auLigne [w] != auLigne [w + 1] ) 
    
    // Niveaux diffrents => un contour
    break ;

// Adresse de la  transition prcdente (initialise  la 1e)
unsigned int  wTransitionPrecedente = w ;

if ( axCellTable ) 
  axCellTable [0] =  (w + 1) / 2.0 ;
  {
  
  // L'adresse exacte de la transition entre w et w+1 est w+1.
  // Bloc scuris
  if ( w >= wLongueurLigne ) 
    
    // 0 transitions ou erreur bizarre.
    goto FinPrematuree ;
  else 
    {
    w++ ;
    pwTablePeriodes [0] =  w ;
    }
  
  // Table des autres transitions.
  unsigned int  wNoTransition = 1 ;
  for ( w ; w < wLongueurLigne - 1 ; w++ ) 
    if ( auLigne [w] != auLigne [w + 1] ) 
      {
      
      // Niveaux diffrents => un contour
      pwTablePeriodes [wNoTransition] =  w - wTransitionPrecedente ;
      if ( axCellTable ) 
        axCellTable [wNoTransition]
          =  (w + wTransitionPrecedente) / 2.0 + 1 ; // gale (wTP+1)+(w-wTP)/2)
      wTransitionPrecedente =  w ;
      wNoTransition++ ;
      }
  
  // Dernire priode. w==wLongueurLigne-1
  pwTablePeriodes [wNoTransition]
    =  wLongueurLigne - 1 - wTransitionPrecedente ;
  if ( axCellTable ) 
    axCellTable [wNoTransition]
      =  (wLongueurLigne + wTransitionPrecedente) / 2.0 + 0.5 ; // gale (wTP+1)+((wLL-1)-wTP)/2)
  unsigned int  wNbPeriodes = wNoTransition + 1 ;

  // Normalisation:
  if ( axCellTable )
    for ( w = 0 ; w < wNbPeriodes ; w++ )
      axCellTable [w] /= wLongueurLigne ;
  if ( wNbPeriodes > wLongueurLigne )

    // Erreur grave et bizarre.
    goto Exception ;
  if ( wNbPeriodes <= 3 )

    // Frquence trop faible (<= 3).
    goto FinPrematuree ;

  //  On fait l'analyse en tant les transitions extrmes.
  statistiques  statTablePeriodes
    = Statistiques_courbe(pwTablePeriodes + 1, wNbPeriodes - 2);
  unsigned int  *pwTablePeriodesCorrigee = pwTablePeriodes ;
  *pRacourcissementF =  *pRacourcissementD = FAUX ;

  //     *pxSeuilRaccourcissement=statTablePeriodes.moy-2.0*statTablePeriodes.ecart_type;
  if ( (wProperties & SMB_PROP_Origin)
        && (wProperties & SMB_PROP_Appearance) )
    {

    // On sait de quel ct est le motif en L *et* quelle est sa couleur
    if ( (wProperties & SMB_PROP_OriginA_D) == SMB_PROP_OriginA ) // Origine  gauche
                                                                    {
  if ( wProperties & SMB_PROP_AppPositive )
    {

    // Origine en noir, il faut auLigne[0]==0
    *pRacourcissementD =  !auLigne [0];
    if ( wProperties & SMB_PROP_Parity )
      //On connat la parit
      *pRacourcissementF
        =  (wProperties & SMB_PROP_Even) ?
             auLigne [wLongueurLigne - 1]
          : !auLigne [wLongueurLigne - 1];
    }
  else
    {

    // Origine en blanc, il faut auLigne[0]=0
    *pRacourcissementD =  auLigne [0];
    if ( wProperties & SMB_PROP_Parity )
      //On connat la parit
      *pRacourcissementF
        =  (wProperties & SMB_PROP_Even) ?
             !auLigne [wLongueurLigne - 1]
          : auLigne [wLongueurLigne - 1];
    }
                                                                    }
    else
      {

      // Origine  droite
      // Il faut auLigne[wLongueurLigne-1] identique  SMB_PROP_AppPositive
      if ( wProperties & SMB_PROP_AppPositive )
        {
        *pRacourcissementF =  !auLigne [wLongueurLigne - 1];
        if ( wProperties & SMB_PROP_Parity )
          //On connat la parit
          *pRacourcissementD
            =  (wProperties & SMB_PROP_Even) ? auLigne [0] : !auLigne [0];
        }
      else
        {

        // Origine en blanc, il faut auLigne[(fin)]==0
        *pRacourcissementF =  auLigne [wLongueurLigne - 1];
        if ( wProperties & SMB_PROP_Parity )
          //On connat la parit
          *pRacourcissementD
            =  (wProperties & SMB_PROP_Even) ? !auLigne [0] : auLigne [0];
        }
      }
    }
  if ( PeriodeAccidentelle(pwTablePeriodes [wNbPeriodes - 1]
        , // Longueur de la dernire priode
          statTablePeriodes.moy
        , statTablePeriodes.ecart_type, pxSeuilRejet) )

    // Dernire priode accidentelle
    *pRacourcissementF =  VRAI ;
  if ( *pRacourcissementF )
    wNbPeriodes-- ;

  //    if (puTablePeriodes[0]<*pxSeuilRaccourcissement)
  /* Calcul pour la premire priode en premier car le racourcissement du dbut
     modifie les indices. */
  if ( PeriodeAccidentelle(pwTablePeriodes [0], statTablePeriodes.moy
        , statTablePeriodes.ecart_type) )

    // Premire priode est accidentelle
    *pRacourcissementD =  VRAI ;

  // On peut faire l'analyse sur la cohrence du signal en crneaux connaissant exactement le nombre de priodes.
  if ( *pRacourcissementD )
    {
    if ( axCellTable )
      ShiftTable(axCellTable, wNbPeriodes, - 1);
    pwTablePeriodesCorrigee++ ;
    wNbPeriodes-- ;
    }
  if ( (wProperties & SMB_PROP_Parity)
        && !*pRacourcissementD
        && !*pRacourcissementF )

    /* On connat la parit et il n'y a pas eu (encore) de racourcissement (le
       racourcissement qu'il aurait peut-tre d y avoir n'a pas t dtect,
       au cause d'une longueur non diffrente, ou parce que l'on ne connat pas
       l'origine ou la couleur.
         Dans ce cas, le racourcissement est fait en aveugle, on ne sait pas s'il
       s'applique au dbut ou  la fin. Il faut alors couper l'alignement sur les
       cellules (cellTable); pour faire a: on foit notifier  la procdure parent
       le racourcissement en aveugle.
       */
    if ( (wProperties & SMB_PROP_Even) && wNbPeriodes % 2 )
      {

      // frequ connue paire mais dtecte impaire
      wNbPeriodes-- ;
      *pfCompatibilite |= ANLFREQ_MSG_RaccAveugle ;
      }
    else if ( !(wProperties & SMB_PROP_Even) && !(wNbPeriodes % 2) )
      {

      // frequ connue impaire mais dtecte paire
      wNbPeriodes-- ;
      *pfCompatibilite |= ANLFREQ_MSG_RaccAveugle ;
      }
  statTablePeriodes
    =  Statistiques_courbe(pwTablePeriodesCorrigee, wNbPeriodes);
  *piFrequence        =  wNbPeriodes < MAXINT ? wNbPeriodes : MAXINT ;
  *pxEcartTypePeriode =  statTablePeriodes.ecart_type ;
  *pwPeriodeMin       =  statTablePeriodes.min ;
  *pwPeriodeMax       =  statTablePeriodes.max ;
  *pxPeriode          =  statTablePeriodes.moy ;
  free(pwTablePeriodes);
  return SUCCES ;
  }
Exception :
    {
    free(pwTablePeriodes);
    return ECHEC ;
    }
FinPrematuree :
    {
    *pfCompatibilite |= ANLFREQ_MSG_NonCompatible ;
    free(pwTablePeriodes);
    return SUCCES ;
    }
  }

/*-------------------------------------------------------------------------*/
// Cette fonction a un effet sur le tableau puLigne.
BOOL SeuilleLigne ( unsigned char *puLigne, unsigned int wLongueurLigne
    , float xValeurSeuil )
{
if ( !puLigne ) 
  return ECHEC ;
else 
  for ( unsigned int w = 0 ; w < wLongueurLigne ; w++ ) 
    puLigne [w] =  puLigne [w] > xValeurSeuil ? 255 : 0 ;
return SUCCES ;
}

/*------------------------------------------------------------------------*/
// Dynamique demande: de 0.0  1.0. 1.0->chec.
BOOL CritereValiditeFrequenceDUneLigne (
    int iFrequence // Frquence suppose
    , float xPeriode // Priode dduite
    , float xEcartTypePeriode // cart type sur la veleur des largeure de cellules
    , unsigned char uPeriodeMin // Plus courte cellule
    , unsigned char uPeriodeMax // Plus longue cellule
    , float *pxCritere )
{
//#define xNormalisation 10.0
if ( !pxCritere || iFrequence <= 0 )
  return ECHEC ;
if ( PeriodeAccidentelle(uPeriodeMin, xPeriode, xEcartTypePeriode)
      || PeriodeAccidentelle(uPeriodeMax, xPeriode, xEcartTypePeriode) )
  *pxCritere =  1.0 ; // ECHEC;
else
  *pxCritere =  xEcartTypePeriode / 10.0 ;
return SUCCES ;
//#undef xNormalisation
} /*-------------------------------------------------------------------------*/ 
