/*******************|*******************************************/
/*-------------------------------------------------------------*/
/*    Fonction       : Fonctions exportes de Symole.C (trai-  */
/*                     ment symbolique de symboles).           */
/*                                                             */
/* Fichier prototype SYMBOLE .H.                               */
/* (contient les prototypes des objets  exporter)             */
/*-------------------------------------------------------------*/
/*    Auteur         : Baptise MARCEL                          */
/*    Dvelopp le   : automne 1995                            */
/***************************************************************/
#ifndef WSYMBOLE_H
#define WSYMBOLE_H

#include "base.h"

#include "scaltool.h"
/*-------------------------------------------------------------*/
/*			   constantes                          */
/*-------------------------------------------------------------*/
#define NombreCaractereMaxSymbole 8192
#define CONST_NoneSymbolMessage "none"

// Bit-wise flags:
#define SMB_CELL_Mark        0xF0 // Cellule marque ou non
#define SMB_CELL_Unknown     0x01 // Incertitude totale
#define SMB_CELL_Error       0x02 // Erreur sur la valeur de la cellule (suppose ou certaine)

#define SMB_CELL_Marked      0xF0
#define SMB_CELL_Unmarked    0x00




#define DM_FP_Left  0
#define DM_FP_Right 4

#define SMB_DISPLAY_PlanImage 0
//#define IMPRESSION  1
//#define ECRAN       2

/*-------------------------------------------------------------*/
/*		    nouveaux types  exporter                  */
/*-------------------------------------------------------------*/
class TMatrix
  {
  protected :
    unsigned int wSize;
    unsigned char *puData;
  public :
    TMatrix(unsigned int w=0); // Matrice nitialise  NULL
    ~TMatrix();
    void Set(unsigned int w=0);
    BOOL Rotation90deg(char cRotat);
    BOOL Transpositon();
    void GenerationSpontanee(unsigned int w=0);
    void Copie(TMatrix *pmat);
    unsigned char SetCell(unsigned char i, unsigned char j, unsigned char c)
      {return puData[i*wSize+j]=c;}
    unsigned char GetCell(unsigned char i, unsigned char j)
      {return puData[i*wSize+j];}
    unsigned int GetSize()
      {return wSize;}
    unsigned char *GetData()
      {return puData;}
  };
class TDataMatrix : public TMatrix
  {
  public :
    TDataMatrix(unsigned int s=0):TMatrix(s){}; // Matrice nitialise  NULL
    ~TDataMatrix(){};
    BOOL Binarisation(unsigned char *acListeCaracteres, int iFreq);
    BOOL Encadrement();
    BOOL PivoteHorlogeRail();
    BOOL TransfertVisu(unsigned char *puCible);
    BOOL MakeOutOfString(char *sz);
    BOOL MakeOutOfWrappedString(char *sz);
    int AnalyseFinderPattern(){return 0;}
  };
/* Rgle d'utilisation :
  Un objet de TDataMatrix est :
    soit nul, c'est  dire wFrequence=0 et Matrice=NULL;
    soit allou, c'est  dire wFrequence != 0 et Matrice pointe sur une
      matrice de taille wFrequence
/*-------------------------------------------------------------*/
/*	            prototypes de variables                    */
/*-------------------------------------------------------------*/
extern char szSymbolDir[MAXDIR];
extern char szSymbolDrive[MAXDRIVE];
/*-------------------------------------------------------------*/
/*	            prototypes de fonctions                    */
/*-------------------------------------------------------------*/
BOOL Octavisation (TDataMatrix *pSymbole, char *catalogos, long n_oct);
BOOL TextualiseListeOctets (char *acListeOctets, float xNbOctets);
BOOL NumeraliseListeOctets (char *szTexteNum, char *acListeOctets, unsigned int wNbOct);
unsigned char CouleurDePoint (unsigned char u, unsigned char uType);
BOOL ChargeSymbole (char *filename, TDataMatrix *smb);
BOOL SauvegardeSymbole(TDataMatrix *pSmb, char *szFileName);

int DeduceSymbolName(char szSymbolName[MAXFILE],char *szImageUtilisee,
  char *szAssociationLibrary);
int FindSymbolFile(char *szName, char *szPath=NULL); // Look in several directories if a symbol match that name.
int SplitSampleSymbolName(char *szImageName,
    char *pcSampleName, // Nom de la srie d'chantillonnage
    unsigned char *puDate, // Date de la srie (No. du mois en hexa)
    unsigned char *puNumber, // Numro du symbole
    char *pcSpin, // Variante (lettre f-g-h...)
    unsigned char *puFreq); // Frquence du symbole

inline unsigned int FrequencePourTantDOctets (unsigned int wOct)
  {
  return 2+NCeilR(sqrt(wOct*8));
  }
inline float NbROctetsPourTelleFrequence (unsigned int wFreq)
  {
  return (wFreq-2)*(wFreq-2)/8.0;
  }
inline unsigned int NbOctetsPourTelleFrequence (unsigned int wFreq)
  {
  return NCeilR(NbROctetsPourTelleFrequence(wFreq));
  }
#endif // WSYMBOLE_H
