/*******************|*******************************************/
/*                  W M A T H D I V . H                        */
/*-------------------------------------------------------------*/
/*    Fonction       : Fonctions mathmatiques.                */
/*                                                             */
/*                                                             */
/* Fichier prototype WMATHDIV.H.                               */
/* (contient les prototypes des objets  exporter)             */
/*-------------------------------------------------------------*/
/*    Auteur         : Baptise MARCEL                          */
/*    Dvelopp le   : t     1994/1995                        */
/***************************************************************/
#ifndef WGEOMETR_H
#define WGEOMETR_H

#include <values.h>
#include <math.h>

#include "base.h"
#include "scaltool.h"

/*-------------------------------------------------------------*/
/*			   constantes                                         */
/*-------------------------------------------------------------*/
#define DROITE_POINT_VECTEUR 1
#define QUAD_ACQUISITION 0
/*-------------------------------------------------------------*/
/*		    nouveaux types  exporter                  */
/*-------------------------------------------------------------*/
typedef struct STATISTIQUES
  {
  unsigned int min;
  unsigned int max;
  long som;
  float med;
  float moy;
  float ecart_type;
  } statistiques;

typedef struct uCOORDONNEE
  {
  unsigned char x;
  unsigned char y;
  } uCoordonnee;

typedef struct LIGNE_AB
  {
  unsigned char lg;
  uCoordonnee A;
  uCoordonnee B;
  } ligne_AB ;

//--------------------------------------------------------------------------
//		        Nouvelles classes  exporter
//-------------------------------------------------------------------------
class TBipoint;
class TLigne;
class TVecteur;
class TDroite;
//class TQuadrilatere;

class TCoordonnee : public TPoint
  {
  public :
//    TCoordonnee(uCoordonnee sM);
    TCoordonnee(TVecteur V);
    TCoordonnee(int _x=0, int _y=0);
    TCoordonnee(TPoint M);
    TCoordonnee(TLigne AB);
    void Set(int _x=0, int _y=0);
    void Set(TPoint p);
    unsigned long int DistanceCarree(TPoint p);
    double Distance(TPoint p);
    unsigned long int ModuleCarre();
    BOOL Existe(); // Par convention (MAXINT,MAXINT) n'existe pas.
  };

class TVecteur
  {
  public :
    float x;
    float y;
    TVecteur(float _x=0.0,float _y=0.0);
//    TVecteur(uCoordonnee ptsM);
    TVecteur(TPoint ptA);
    TVecteur(TVecteur ptxA, TVecteur ptxB);
    TVecteur(TPoint ptA, TPoint ptB) {x=ptB.x-ptA.x;y=ptB.y-ptA.y;}
//    TVecteur(ligne_AB AB);
    TVecteur(TLigne AB);
    TVecteur(TRect AB){x=AB.right-AB.left;y=AB.bottom-AB.top;};
      // A=TopLeft, B=LowRight. //x=B.x-A.x;
                                //  y=B.y-A.y;

    void Set(float _x=0.0, float _y=0.0);
    char EstNul();
    float Module();
    float ModuleCarre();
    float Orientation();
    float DistanceCarree(TVecteur ptxU);  // Distance entre deux points
    char AppartientDroite(TDroite D);  // Appartenance point-droite
    double Distance(TDroite D);         // Distance point-droite
    double DistanceAlgebriqueDroite(TVecteur ptxA, TVecteur ptxB);
    float DistanceCarree(TDroite D);   // Distance carre point-droite
    double DistanceCarreeAlgebriqueDroite(TVecteur ptxA, TVecteur ptxB);
    BOOL Existe();
    char DroiteOuGaucheDroite(TPoint ptA, TPoint ptB);
    char DroiteOuGaucheDroite(TVecteur ptxA, TVecteur ptxB);
    static TVecteur Barycentre (TVecteur ptxA, float a, TVecteur ptxB, float b);
    TVecteur Unitaire(); // Le mme vecteur divis par sa norme.
    TVecteur Pivote(float xAlpha);    // Rotation vectorielle ou rotation autour du centre du repere.
    TVecteur Pivote(TVecteur xyCentre, double dAlpha); // Rotation autour d'un point.
  };

class TBipoint : public TRect
  {
  public :
    inline TBipoint(TCoordonnee A, TCoordonnee B):TRect((TPoint)A,(TPoint)B){}
    inline TBipoint(TVecteur A,TVecteur B):TRect(TCoordonnee(A),TCoordonnee(B)){}
    inline TBipoint(int left_in,int  top_in,int  right_in,int  bottom_in):TRect(left_in, top_in, right_in, bottom_in){}
    inline TBipoint():TRect(){}
    TBipoint(TLigne ligne);
    inline TBipoint(TRect r):TRect(r){};
    BOOL Existe();
  };

class TLigne
  {
  public :
    TCoordonnee A;
    TCoordonnee B;
    unsigned char lg;
    TLigne ();
//    TLigne (ligne_AB AB);
    TLigne (TRect r){A=r.TopLeft();B=r.BottomRight();} // Remplis lg  0.
    TLigne (TPoint A_in, TPoint B_in, unsigned char u=0); // Remplis lg  u.
    TLigne(TVecteur A_in, TVecteur B_in, unsigned char u=0);
    unsigned long LongueurCarree();
    double Longueur();
    BOOL EstNulle();
    BOOL Existe(); // Ne vrifie pas qu'lg a une bonne valeur.
  };

class TVecteur3
  {
  public :
    float a;
    float b;
    float c;
    TVecteur3(float a_in=0.0, float b_in=0.0, float c_in=0.0);
    char EstNul();
    void Set(float a_in=0.0, float b_in=0.0, float c_in=0.0);
    void OrdonneCroissant();
  };

class TDroite : public TVecteur3
  {
  public :
    TDroite(TBipoint bip);
    TDroite(TVecteur vA, TVecteur vB, char cMode=0);
    TDroite(TPoint A, TPoint B, char cMode=0);
    TDroite(float a_in=MAXFLOAT, float b_in=MAXFLOAT, float c_in=0.0);
    double Distance(TVecteur v);
    double DistanceAlgebrique(TVecteur v);
    float DistanceCarree(TVecteur v);
    float DistanceCarreeSignee(TVecteur v);
    int Existe(){return (a != MAXFLOAT && b!=MAXFLOAT && c!=MAXFLOAT && (a !=0.0 || b!=0.0));}
    TVecteur Intersection(TDroite D);
    static TDroite Horizontale(float y=0.0);
    static TDroite Verticale(float x=0.0);
    BOOL Paralleles (TDroite D);
    BOOL FaiblePente();
    BOOL FortePente();
    BOOL PlusRaide(TDroite dr);
    BOOL MoinsRaide(TDroite dr);
    BOOL EstHorizontale();
    BOOL EstVerticale();
    BOOL EstDroiteEtPasOblique();
    static BOOL CalculeFuiteFaisceau(TDroite AB, TDroite CD,
        TVecteur *Pf_AB, int *AB_converge);
    TVecteur VecteurPorteur();
    TVecteur PointMembre();
    TDroite Pivote(double dAlpha);
    TDroite Pivote(TVecteur xyOmega, double dAlpha);
    BOOL Bissectrice(TDroite dr2, TDroite *drBiss1, TDroite *drBiss2);
    TVecteur ProjectionPtVect(TVecteur xyM, TVecteur xyu)
      {return Intersection(TDroite(xyM,xyu,DROITE_POINT_VECTEUR));}
  };

class TQuadrilatere
  {
  public :
    TCoordonnee A;
    TCoordonnee B;
    TCoordonnee C;
    TCoordonnee D;
    TQuadrilatere(TPoint A_in=TPoint(), TPoint B_in=TPoint(), TPoint C_in=TPoint(),TPoint D_in=TPoint());
    TQuadrilatere(const TQuadrilatere& ABCD_in);
    TQuadrilatere(TRect r);
    void Set(TPoint A_in=TPoint(), TPoint B_in=TPoint(), TPoint C_in=TPoint(),TPoint D_in=TPoint());
    void Set(TQuadrilatere ABCD_in);
    void Set(TRect r);
    BOOL Pervers();
    BOOL Existe();
    BOOL DroitesPorteuses
      (TDroite *pAB, TDroite *pBC, TDroite *pCD, TDroite *pAD);
    BOOL DecCircMaxEnTete(int *pi=NULL);
    int Flip();
    int FlipMaxEnSecond(BOOL fCondition=TRUE);
    int IsParcoursTrigo(){return TVecteur(B).DroiteOuGaucheDroite(A,C)==DROITE;}
    int FlipParcoursTrigo();
    BOOL DecCirc(int i=1);
    BOOL Plat();
    double Circonference()
      {
      return A.Distance(B)+B.Distance(C)+C.Distance(D)+D.Distance(A);
      }
    inline void ReOrgQuadMaxMax(int *pi=NULL)
/* renvoit la transformation effectue: *pi=i;
 abs(i): nb de DecCirc (4 => pas de DecCirc) puis i<0 => FLIPED;
 */
      {
      DecCircMaxEnTete(pi);
      if (pi && ! *pi)
        *pi +=4;
      if (FlipMaxEnSecond() && *pi)
        *pi *= -1;
      }
    BOOL Convexe();
    BOOL Croise();
    BOOL EstDroitEtPasOblique();
    TVecteur Centre(); //Renvoit l'intersection des diagonale.
    TRect RectangleCirconscrit()
      {
      return TRect(min(A.x,B.x,C.x,D.x),
                   min(A.y,B.y,C.y,D.y),
                   max(A.x,B.x,C.x,D.x),
                   max(A.y,B.y,C.y,D.y)
                   );
      }
  };
// log
//--------------------------------------------------------------------------
//	                 prototypes de variables
//-------------------------------------------------------------------------
extern TDroite DroiteErreur, drPremiereBissectrice, drDeuxiemeBissectrice;
extern TCoordonnee CoordonneeErreur;
extern TVecteur VecteurNul;
extern TVecteur VecteurErreur;
extern ligne_AB sLigne_00;
extern TQuadrilatere QuadrilatereErreur;
extern TLigne LigneErreur;
extern TBipoint bipNul, bipErreur;
;
//--------------------------------------------------------------------------
//	                 prototypes de fonctions
//-------------------------------------------------------------------------
//-------------------------------------------------------------------------
/*	                   fonctions de contrle                    */
char *Item2String(TQuadrilatere q, char *sz);
char *Item2String(TLigne l, char *sz);
char *Item2String(TVecteur3 xyz, char *sz);

/*	                   fonctions scalaires                      */
/*	                   fonctions sur courbes                    */
unsigned char Loc_maximum_courbe (unsigned char *courbe, unsigned int l);
unsigned char Loc_minimum_courbe (unsigned char *courbe, unsigned int l);
unsigned char Minimum_courbe (unsigned char *x, long taille);
unsigned char Maximum_courbe (unsigned char *x, long taille);
unsigned char Somme_courbe (unsigned char *x, long taille);
char Courbe_plate (unsigned char *x, long taille);
float Mediane_courbe (unsigned char *x, long taille);
float Ecart_type_courbe (unsigned char *x, long taille);
statistiques Statistiques_courbe (unsigned int *x, long taille);
statistiques Statistiques_courbe (unsigned char *x, long taille);

/*	                   fonctions pour images                    */
char ImageAbsGrad2x2(unsigned char *source, unsigned char *destin);
int AbsGradiant2x2(unsigned char i, unsigned char j, unsigned char *img);

/*	                   fonctions sur matrices                   */
char Transposition_MAT(unsigned char *mat1, unsigned char *mat2, unsigned long m, unsigned long n);
//char Rotation_mod90_MAT(unsigned char *mat1, unsigned char *mat2, int n, char rot);
/*	                   fonctions pour points (coordonnees)       */

char operator ==(uCoordonnee A,uCoordonnee B);
uCoordonnee Cv_uCoordonnee (TCoordonnee M);
uCoordonnee Cv_uCoordonnee (TVecteur U);

TCoordonnee operator +(TCoordonnee A,TCoordonnee B);
TCoordonnee operator -(TCoordonnee A,TCoordonnee B);

/*	                   fonctions pour points (vecteurs)          */
int operator ==(TVecteur A, TVecteur B);
TVecteur operator +(TVecteur U1, TVecteur U2);
TVecteur operator -(TVecteur U1, TVecteur U2);
TVecteur operator /(TVecteur U, float x);
TVecteur operator *(TVecteur U, float x);
float operator *(TVecteur U1, TVecteur U2); // Produit scalaire
float operator ^(TVecteur U1, TVecteur U2); // Produit vectoriel
double Angle(TVecteur U1, TVecteur U2);
double cosAngle(TVecteur U1, TVecteur U2);
double sinAngle(TVecteur U1, TVecteur U2);
double tanAngle(TVecteur U1, TVecteur U2);

/*	                   fonctions sur bipoints                   */
BOOL ExisteRect(TRect r);
ligne_AB Cv_ligne (TLigne l);

/*	                   fonctions pour droites                   */

/*	                   fonctions pour quadrilateres             */
BOOL WritePrivateQuad(char *szSection, char * szEntry,
    TQuadrilatere qValue, char *szFileName);
TQuadrilatere GetPrivateQuad(char *szSection, char * szEntry,
    TQuadrilatere qDefault, char *szFileName);
TQuadrilatere ReOrgQuadMaxMax(TQuadrilatere ABCD);

inline TCoordonnee operator /(TPoint pt, double x)
  {
  return TCoordonnee(TVecteur(pt.x/x,pt.y/x));
  }

inline TQuadrilatere operator /(TQuadrilatere q, double x)
  {
  q.A =q.A/x;q.B =q.B/x;q.C =q.C/x;q.D =q.D/x;
  return q;
//  NFarRoundR((TPoint)x;  q.B /= x; q.C /= x; q.D /= x;
  }


/*	                   fonctions gomtriques spcialise         */
char TypeTriangle3longu(float AB, float BC, float CA);
char AireTriangle3longu(float AB, float BC, float CA, float *P_aire);
char TrianglePlat (TVecteur A, TVecteur B, TVecteur C);
BOOL AnglesTriangle3longu(float AB, float BC, float CA, TVecteur3 *PABC);

//	                   fonctions pour polygones
BOOL GetPrivatePolygon(TCoordonnee *aptPolygon,
    unsigned char uNbPoints, char *szSection, char * szEntry,
    TPoint ptDefault, char *szFileName);
BOOL WritePrivatePolygon(char *szSection, char * szEntry,
    TCoordonnee *aptPolygon, unsigned char uNbPoints, char *szFileName);
BOOL PolygoneExiste(TCoordonnee *aptPolygone, unsigned char uNbPolygon);
TRect RectangleCirconscrit(TCoordonnee *aptPolygone,unsigned char uNbPoints);

extern TRect rImage, rImageBorder;


// Fonctions inline diverses
inline BOOL ExisteRect(TRect r)
  {
  return TBipoint(r).Existe();
  }
inline unsigned int SizeOfOutput(TQuadrilatere &)
  {
  return 64;
  }
inline unsigned int SizeOfOutput(TLigne &)
  {
  unsigned char u;
  return 2*SizeOfOutput(TPoint())+SizeOfOutput(u)+7;
  }
inline unsigned int SizeOfOutput(TVecteur3 &)
  {
  return 128; // estimation ~ pif
  }

// Fonctions inline TCoordonnes
inline void TCoordonnee::Set(TPoint p)
  {
  x=p.x;y=p.y;
  }
inline TCoordonnee::TCoordonnee(TPoint M):TPoint(M.x,M.y){}
inline BOOL TCoordonnee::Existe()
  {
  return x!=MAXINT && y != MAXINT;
  }
inline char operator ==(uCoordonnee A,uCoordonnee B)
  {
  return A.x==B.x && A.y==B.y;
  }
inline unsigned long int TCoordonnee::DistanceCarree(TPoint p)
  {
  return (unsigned long)(p.x-x)*(p.x-x)+(unsigned long)(p.y-y)*(p.y-y);
  }
inline double  TCoordonnee::Distance(TPoint p)
  {
  return sqrt((unsigned long)(p.x-x)*(p.x-x)+(unsigned long)(p.y-y)*(p.y-y));
  }
inline void TCoordonnee::Set(int _x, int _y)
  {
  x=_x;
  y=_y;
  }
inline TCoordonnee::TCoordonnee(int _x, int _y):TPoint(_x,_y)
  {
  }
inline TCoordonnee Cv_TCoordonnee(uCoordonnee sM){return TCoordonnee (sM.x,sM.y);}
inline TCoordonnee::TCoordonnee(TLigne AB) : TPoint (TCoordonnee(AB.B)-TCoordonnee(AB.A))
  {
  }
inline TCoordonnee::TCoordonnee(TVecteur V):TPoint(NFarRoundR(V.x),NFarRoundR(V.y)){}

inline unsigned long int TCoordonnee::ModuleCarre()
  {
  return x*x+y*y;
  }
inline TCoordonnee operator +(TCoordonnee A,TCoordonnee B)
  {
  return TCoordonnee(A.x+B.x,A.y+B.y);
  }
inline TCoordonnee operator -(TCoordonnee A,TCoordonnee B)
  {
  return TCoordonnee(A.x-B.x,A.y-B.y);
  }

// Fonctions inline TVecteur
inline TVecteur::TVecteur(TLigne AB){x=AB.B.x-AB.A.x;y=AB.B.y-AB.A.y;}
inline TVecteur::TVecteur(TPoint A)
  {
  x=A.x;
  y=A.y;
  }
inline TVecteur::TVecteur(TVecteur A, TVecteur B)
  {
  x=B.x-A.x;
  y=B.y-A.y;
  }
inline char TVecteur::DroiteOuGaucheDroite(TPoint ptA, TPoint ptB)
  {
  return DroiteOuGaucheDroite(TVecteur(ptA), TVecteur(ptB));
  }
inline TVecteur::TVecteur(float _x,float _y)
  {
  x=_x;
  y=_y;
  }
inline void TVecteur::Set(float _x,float _y)
  {
  x=_x;
  y=_y;
  }
/*
  inline TVecteur::TVecteur(uCoordonnee sM)
  {
  x=sM.x;
  y=sM.y;
  }
*/
inline char TVecteur::EstNul()
  {
  return (ModuleCarre()<epsilon*epsilon);
  }
inline float TVecteur::Module()
  {
  return sqrt(ModuleCarre());
  }
inline float TVecteur::ModuleCarre()
  {
  return x*x+y*y;
  }
inline float TVecteur::DistanceCarree(TVecteur U)
  {
  return (U.x-x)*(U.x-x)+(U.y-y)*(U.y-y);
  }
inline TVecteur operator +(TVecteur U1, TVecteur U2)
  {
  return TVecteur(U1.x+U2.x,U1.y+U2.y);
  }
inline TVecteur operator -(TVecteur U1, TVecteur U2)
  {
  return TVecteur(U1.x-U2.x,U1.y-U2.y);
  }
inline TVecteur operator *(TVecteur U, float x)
  {
  return TVecteur(U.x * x,U.y * x);
  }
inline TVecteur operator /(TVecteur U, float x)
  {
  return TVecteur(U.x/x,U.y/x);
  }
inline float operator *(TVecteur U1, TVecteur U2) // Produit scalaire
  {
  return U1.x*U2.x+U1.y*U2.y;
  }
inline float operator ^(TVecteur U1, TVecteur U2) // Produit vectoriel
  {
  return U1.x*U2.y-U2.x*U1.y;
  }
/*
inline TVecteur::TVecteur(ligne_AB AB)
  {
  *this=TVecteur(AB.B)-TVecteur(AB.A);
  }
*/
/*
inline TVecteur::TVecteur(TLigne AB)
  {
  *this=TVecteur(AB.B)-TVecteur(AB.A);
  }
*/
inline int operator ==(TVecteur A, TVecteur B)
  {
  return A.DistanceCarree(B)<epsilon*epsilon;
  }
inline BOOL TVecteur::Existe()
  {
  return x!=MAXFLOAT && y!=MAXFLOAT;
  }
inline float TVecteur::Orientation()
  {
    return EstNul() ? MAXFLOAT : (float)atan2(y,x);
  }

// Fonctions inline TBipoint
inline TBipoint::TBipoint(TLigne ligne):TRect(ligne.A,ligne.B){} // UpLeft and LowRight

// Fonctions inline TLigne
inline TLigne::TLigne ():A(0,0),B(0,0)
  {
  }
inline BOOL TLigne::EstNulle()
  {
  return A==B;
  }
inline unsigned long int TLigne::LongueurCarree()
  {
  return TCoordonnee(*this).ModuleCarre();
  }
inline BOOL TLigne::Existe() // Ne vrifie pas qu'lg a une bonne valeur.
  {
  return A.Existe() && B.Existe();
  }

/*
inline TLigne::TLigne(ligne_AB AB):A(AB.A),B(AB.B)
  {
  lg=AB.lg;
  }
*/
inline TLigne::TLigne (TPoint A_in, TPoint B_in, unsigned char u):A(A_in),B(B_in)
  {
  lg=u;
  }
inline TLigne::TLigne(TVecteur A_in, TVecteur B_in, unsigned char u):A(A_in),B(B_in)
  {
  lg=u;
  }
inline TLigne Cv_TLigne(ligne_AB lAB)
  {return TLigne(Cv_TCoordonnee(lAB.A),Cv_TCoordonnee(lAB.B),lAB.lg);}

// Fonctions inline TDroite
inline BOOL TDroite::FaiblePente()
    {
// Pente: -a/b, |pente|=|a|/|b|, faible pente <=> |a|/|b| <= 1 <=>  |a| <= |b|
    return fabs(a) <= fabs(b);
    }
inline BOOL TDroite::FortePente()
    {
    return fabs(a) > fabs(b);
    }

inline TDroite::TDroite(float a_in, float b_in, float c_in):TVecteur3(a_in,b_in,c_in){}
inline TDroite TDroite::Horizontale(float y)
  {
  return TDroite(0.0,1.0,-y);
  }
inline TDroite TDroite::Verticale(float x)
  {
  return TDroite(1.0,0.0,-x);
  }
inline TDroite::TDroite(TVecteur A, TVecteur B, char cMode)
           :TVecteur3(B.y,-B.x,- B.y * A.x + B.x * A.y)
  {
   // !=DROITE_POINT_VECTEUR => 2 points
  if (cMode)
    c=B.x * A.y- B.y * A.x;
  else // cMode != DROITE_POINT_VECTEUR => 2 points
    {
    a=B.y - A.y;
    b=A.x - B.x;
    c=- A.x * a - A.y * b;
    }
  }
inline TDroite::TDroite(TBipoint bip)
  :TVecteur3(bip.bottom - bip.top,bip.left- bip.right,0)
  {c=- bip.left * a - bip.top * b;}

inline TDroite::TDroite(TPoint A, TPoint B, char cMode)
           :TVecteur3(B.y,-B.x,0)
  {
  if (cMode)
    c=B.x * A.y- B.y * A.x;
  else // cMode != DROITE_POINT_VECTEUR => 2 points
    {
    a=B.y - A.y;
    b=A.x - B.x;
    c=- A.x * a - A.y * b;
    }
  }
/*
// Template qui calcule la moyenne.
// ATTENTION. Ne fonctionne que dans le type courant.
// Une table de 'char' aura sa moyenne calcule dans l'espace des 'char',
// Avec notament une somme limite  128!!!
// Interet de cette classe: realiser une moyenne avec des objets quelconques: TVecteur, TDroit...
Tant que les operations suivan,tes existent pour la classe T:
  casting 0->T
  Affectation (existe toujours)
  Addition (T+T -> T)
  Division par unsigned int (T/UINT -> T)
*/
template <class T, class wT> int TableMean(T *a, wT wSize, T *pMean)
  {
  if (!a)
    return FAILURE;
  unsigned long dw;
  *pMean=(T)0;

  for(dw=0;dw<wSize;dw++)
    *pMean = *pMean + a[(unsigned int)dw];
  *pMean=*pMean/wSize;
  return wSize;
  }

// fonctions inline Quadrilatres
inline TQuadrilatere ReOrgQuadMaxMax(TQuadrilatere ABCD)
  {
  ABCD.ReOrgQuadMaxMax();
  return ABCD;
  }
inline TVecteur TQuadrilatere::Centre()
  {
    return TDroite(A,C).Intersection(TDroite(B,D));
  }

inline TQuadrilatere::TQuadrilatere(const TQuadrilatere& ABCD_in)
                  : A(ABCD_in.A),B(ABCD_in.B),C(ABCD_in.C),D(ABCD_in.D)
  {
  }

inline TQuadrilatere::TQuadrilatere(TPoint A_in, TPoint B_in,
                             TPoint C_in,TPoint D_in)
                  : A(A_in),B(B_in),C(C_in),D(D_in)
  {
  }
inline TQuadrilatere::TQuadrilatere(TRect r)
                  : A(r.TopLeft()),B(r.TopRight()),C(r.BottomRight()),D(r.BottomLeft())
  {
  }
inline void TQuadrilatere::Set(TPoint A_in, TPoint B_in,
                             TPoint C_in,TPoint D_in)
  {
  A=A_in;
  B=B_in;
  C=C_in;
  D=D_in;
  }
inline void TQuadrilatere::Set(TQuadrilatere ABCD_in)
  {
  A=ABCD_in.A;
  B=ABCD_in.B;
  C=ABCD_in.C;
  D=ABCD_in.D;
  }
inline void TQuadrilatere::Set(TRect r)
  {
  A=r.TopLeft();
  B=r.TopRight();
  C=r.BottomRight();
  D=r.BottomLeft();
  }

// Templates Matrices
/*-------------------------------------------------------------*/
/* Rotation_mod90_MAT (Rotation modulo 90 de pTargetrice.
  Pivote une matrice carre de taille wSize*wSize autour de son centre de "rot" fois 90.
  */

template <class T> int RotationMatriceMod90(T *pSource, T *pTarget,
    unsigned int wSize, int iRot)
  {
  int i, j, i2, j2;
  float x0=(wSize-1)/2.0, y0=x0, x, y, x2, y2;
  char cost, sint;

  if (pSource==NULL || pTarget==NULL)
    return ECHEC;
// INT_MAX/4*4 est le plus grand int multiple de 4
  switch ((unsigned char)(iRot+ INT_MAX/4*4 )%4)
    {
  case 0 :
    cost=1;
    sint=0;
    break;
  case 1 :
    cost=0;
    sint=1;
    break;
  case 2 :
    cost=-1;
    sint=0;
    break;
  case 3 :
    cost=0;
    sint=-1;
    break;
  default :
    return MErreur("Erreur code",220);
    }
  for (i=0;i<wSize;i++)
    for (j=0;j<wSize;j++)
      {
      x=j-x0;
      y=-i+y0;
      x2=cost*x-sint*y;
      y2=sint*x+cost*y;
      i2=NFarRoundR(y0-y2);
      j2=NFarRoundR(x2+x0);
      pTarget[i2*wSize+j2]=pSource[i*wSize+j];
      }
  return SUCCES;
  }
#endif // WGEOMETR_H

