/*
#include "myowlall.h"
#include "baptiall.h"
#include "cattoall.h"

#pragma hdrstop
*/


#define MODULE_NAME "LSEUBIN"
 /*************************************************************
     L S E U B I N . C
    -------------------------------------------------------------
     Fonction       :   Seuillage binaire automatique
    -------------------------------------------------------------
     Auteur         : MC (d'aprs PELLEGRINO-FABRE TSI94)
     Dvelopp le   : 13/03/95
     Dernire modif.: 28/04/95
    **************************************************************/

//#include "wgttsi.h"
#include "lseubin.h"
#include "sysctr.h"

#define SEUIL_BIMODALITE 0.4
 /*----------------------------------------------
     Fonctions prototypes

    ----------------------------------------------*/

void          Moyenne_Histo (unsigned int histo []) ;
unsigned int  Liste_Sommet (unsigned int histo [256]
  , unsigned int Smt [256]) ;

/*-------------------------------------------------------------
   Procdures du programme
   */
/***************************************************************
   * 	Fonction:  Cal_seuil
   *  	But : dtermine la valeur du seuil optimum de l'image source
   *	      	   selon des critres gomtriques
   *    Entre :     buffer histo
   *   	Sortie:	   valeur du seuil dans tous les cas
   *		   	   ngative si l'image n'est pas bimodale
   ****************************************************************/
int far Cal_seuil ( unsigned int bufhist [256] )
{
int           Seuil, res ;
unsigned int  i, j, N, Smax = 0, Seuil2, Max, Min ;
long          Surf, lSurfmax, tmp ;
float         Sr, Sref, a, b, critere ;
unsigned int  histo [256], Smt [256];

for ( i = 0 ; i < 256 ; i++ ) 
  histo [i] =  bufhist [i];
//recopie buf.histo
Moyenne_Histo(histo);
N =  Liste_Sommet(histo, Smt);
if ( N > 1 ) 
  {
  
  /* Recherche du sommet maximum */
  for ( i = 0 ; i < N ; i++ ) 
    {
    if ( histo [Smt [i]] > histo [Smax] ) 
      Smax =  Smt [i];
    }
  lSurfmax =  LONG_MIN ;
  
  /* Calcul du critre pour tous les sommets autres que Smax */
  for ( i = 0 ; i < N ; i++ ) 
    {
    if ( Smt [i] != Smax ) 
      {
      if ( Smt [i] > Smax ) 
        {
        Max =  Smt [i];
        Min =  Smax ;
        }
      else 
        {
        Min =  Smt [i];
        Max =  Smax ;
        }
      
      /* CALCUL DE LA SURFACE SOUS LA LIGNE histo[Seuil2] */
      Surf =  0 ;
      for ( j = Min ; j < Max ; j++ ) 
        {
        tmp =  ((long)histo [Smt [i]] - (long)histo [j]);
        if ( tmp > 0 ) 
          Surf += tmp ;
        }
      
      /* Cas o le critre est meilleur que pour les prcdents sommets */
      if ( Surf > lSurfmax ) 
        {
        lSurfmax =  Surf ;
        Seuil2   =  Smt [i];
        }
      }
    }
  Seuil =  (Smax + Seuil2) / 2 ;
  
  /* Test de bimodalit */
  if ( Seuil2 > Smax ) 
    {
    Max =  Seuil2 ;
    Min =  Smax ;
    }
  else 
    {
    Min =  Seuil2 ;
    Max =  Smax ;
    }
  
  /* Calcul de la surface du trapze */
  Sref 
    =  ((float)histo [Min] + (float)histo [Max])
      * ((float)Max - (float)Min) / 2 ;
  a    =  ((float)histo [Max] - (float)histo [Min])
    / ((float)Max - (float)Min);
  b    =  (float)histo [Min] - a * (float)Min ;
  
  /* Calcul de la surface de la valle */
  Sr   =  0 ;
  for ( j = Min ; j < Max ; j++ ) 
    {
    Sr += a * j + b - histo [j];
    }
  critere =  Sr / Sref ;
  if ( critere > SEUIL_BIMODALITE ) 
    res =  1 ; /* l'image est bimodale */ 
  else 
    res =  - 1 ; /* l'image n'est pas bimodale */ 
  }
else 
  {
  
  /* il y a un seul sommet : image non bimodale */
  res =  - 1 ;
  if ( Smt [0] < 128 ) 
    Seuil =  (255 + Smt [0]) / 2 ;
  else 
    Seuil =  Smt [0] / 2 ;
  }
return (res * Seuil);
}

/***************************************************************
   *	Fonction:  Moyenne_Histo	            	       		   *
   *	But:   Filtrage de l'histogramme par moyenne de 4 raies   *
   ****************************************************************/
void Moyenne_Histo ( unsigned int histo [] )
{
unsigned int  i ;
unsigned int  histot [256];

/* TRAITEMENT ANTI-SATURATION */
/* dtection d'une image binaire */
if ( (histo [254] != 0) || (histo [1] != 0) ) 
  {
  
  /* l'image n'est pas binaire on supprime la saturation */
  histo [255] =  histo [254];
  histo [0]   =  histo [1];
  }
for ( i = 0 ; i < 253 ; i++ ) 
  {
  histot [i]
    =  (histo [i] + histo [i + 1] + histo [i + 2] + histo [i + 3]) / 4 ;
  }

/* les valeurs limites sont traites  part */
histot [253] =  (histo [253] + histo [254] + histo [255]) / 3 ;
histot [254] =  (histo [254] + histo [255]) / 2 ;
histot [255] =  (histo [255] + histo [254]) / 2 ;

/* On recopie l'histogramme liss dans le tableau d'origine */
for ( i = 0 ; i < 256 ; i++ ) 
  {
  histo [i] =  histot [i];
  }
}

/***************************************************************
   *	Fonction:  Liste_Sommet
   *	But:	   Slectionne les diffrents sommets de l'histo
   *	Sortie:    nombre de sommets
   ****************************************************************/
unsigned int Liste_Sommet ( unsigned int histo [256]
    , unsigned int Smt [256] )
{
unsigned int  i, NbS = 0 ;
int           iPlateau ;

/* Initialisation du tableau  zro */
for ( i = 0 ; i < 256 ; i++ ) 
  Smt [i] =  0 ;

/* La variable iPlateau indique si on est sur un plateau si c'est le cas,elle
   contient l' abscisse du dbut du plateau sinon elle est ngative */
iPlateau =  - 1 ;
for ( i = 0 ; i < 255 ; i++ ) 
  {
  
  /* Cas o la valeur courante est un sommet sans ambiguit: les valeurs
     prcdente et suivante sont plus petites */
  if ( (histo [i + 1] < histo [i]) && (!i || histo [i] > histo [i - 1]) ) /* Le 2e terme est vrai tout
                                                                              le temps si i=0. */ 
{

/* On comptabilise ce sommet */
Smt [NbS] =  i ;
NbS++ ;
iPlateau -= 1 ;
} 
  else 
    {
    
    /* Cas o on entre sur un plateau en montant (le cas en descendant ne
       nous intresse pas) */
    if ( (!i || histo [i] > histo [i - 1])
          && (histo [i + 1] == histo [i])
          && (iPlateau < 0) ) /* Le 1e terme est vrai tout le temps si i=0. */ 
                                {
                                
                                /* On mmorise o a dbut ce plateau */
                                iPlateau =  i ;
                                }
    else 
      {
      
      /* Cas o on sort d'un plateau en descendant On vient donc de
         rencontrer un sommet */
      if ( (histo [i + 1] < histo [i]) && (iPlateau >= 0) ) 
        {
        
        /* On considre que le sommet est  la mi-distance entre la fin et
           le dbut du plateau */
        Smt [NbS] =  (i + iPlateau) / 2 ;
        NbS++ ;
        
        /* iPlateau est remise  jour */
        iPlateau -= 1 ;
        }
      else 
        {
        
        /* Cas o on sort d'un plateau en montant il ne peut y avoir eu un
           sommet sur ce prcdent plateau */
        if ( (histo [i + 1] > histo [i]) && (iPlateau > 0) ) 
          
          /* iPlateau est remise  jour */
          iPlateau -= 1 ;
        }
      }
    }
  }

/* Cas du dernier niveau de gris */
if ( histo [255] > histo [254] ) /* Cas sans ambiguit */ 
                                  {
                                  Smt [NbS] =  255 ;
                                  NbS++ ;
                                  }
else 
  {
  
  /* Cas o on tait sur un plateau */
  if ( (histo [255] == histo [254]) && (iPlateau > 0) ) 
    {
    
    /* On considre qu'il ya eu un sommet */
    Smt [NbS] =  (255 + iPlateau) / 2 ;
    NbS++ ;
    }
  }
return (NbS);
}

/*-------------------------------------------------------------------------
    Fonction :                   CalHistLoc
   -------------------------------------------------------------------------
    Fonction : Calcul d'histogramme d'une fentre du plan source
    Paramtres en entre : bufhist,bufrepa,pointeur image, fensource
    Valeur retourne : Aucune
   -------------------------------------------------------------------------*/
void CalHistLoc ( unsigned int bufhist [256], unsigned int bufrepa [256]
    , unsigned char *puImage, TRect rSource )
{
unsigned int  i, j, b = 0 ;

if ( rSource.IsEmpty() ) 
  {
  MErreur("Fenetre donne non conforme.", 3870);
  return ;
  }
for ( i = 0 ; i < 256 ; i++ ) 
  bufhist [i] =  0 ; /* eff. zone histo */ 
for ( i = rSource.top ; i <= rSource.bottom ; i++ ) 
  for ( j = rSource.left ; j <= rSource.right ; j++ ) 
    (bufhist [(unsigned char)(puImage [(i << 8) + j])])++ ; /* const. f.rpartition */ 
for ( i = 0 ; i < 256 ; i++ ) 
  {
  b           += bufhist [i];
  bufrepa [i] =  b ;
  }
} // **************************************************************************
 /*
    Pour Mmoire: l'appel:
      case 45:
    //   Seuillage local.
        if (modcoul == 0)
        {
          if (!Fenetres_egales(fen = AcqFen(), fenulle))
          {
    	CalHistLoc(bufhist, bufrepa, source, fen);
    	seuil = Cal_seuil(bufhist);
    	if (seuil >= 0)
    	{
    	  Seuillage(seuil);
    	  Transfert(destin);
    	  flag = 1;
    	}
    	else
    	{
    	  Seuillage(-seuil);
    	  Transfert(destin);
    	  flag = 1;
    	  Erreur_gtr("L'Histogramme de la fenetre n'est pas conforme.", 369, INTEMPESTIF);
    	}
          }
        }
        else
          Seg_redui();
        break;
    */ 
