#include "myowlall.h"
#include "baptiall.h"
#pragma hdrstop

/*
 * Generic Convex Polygon Scan Conversion and Clipping
 * by Paul Heckbert
 * from "Graphics Gems", Academic Press, 1990
 */

/*
 * poly_scan.c: point-sampled scan conversion of convex polygons
 *
 * Paul Heckbert	1985, Dec 1989
 */

/*
#include <stdio.h>
#include <math.h>
*/
#include "poly.h"

#include <owl/point.h>
#include <osl/geometry.h>


#define POLY_NMAX 4		/* max #sides to a polygon; change if needed */

typedef struct {		/* A POLYGON VERTEX */
    double sx, sy, sz, sw;	/* screen space position (sometimes homo.) */
    double x, y, z;		/* world space position */
    double u, v, q;		/* texture position (sometimes homogeneous) */
    double r, g, b;		/* (red,green,blue) color */
    double nx, ny, nz;		/* world space normal vector */
} Poly_vert;
/* update poly.c if you change this structure */

typedef struct {		/* A POLYGON */
    int n;			/* number of sides */
    int mask;			/* interpolation mask for vertex elems */
    Poly_vert vert[POLY_NMAX];	/* vertices */
} Poly;
/*
 * mask is an interpolation mask whose kth bit indicates whether the kth
 * double in a Poly_vert is relevant.
 * For example, if the valid attributes are sx, sy, and sz, then set
 *	mask = POLY_MASK(sx) | POLY_MASK(sy) | POLY_MASK(sz);
 */

typedef struct {		/* A BOX (TYPICALLY IN SCREEN SPACE) */
    double x0, x1;		/* left and right */
    double y0, y1;		/* top and bottom */
    double z0, z1;		/* near and far */
} Poly_box;

/*
typedef struct {	//	/* WINDOW: A DISCRETE 2-D RECTANGLE
    int x0, y0;		//	/* xmin and ymin
    int x1, y1;		//	/* xmax and ymax (inclusive)
} Window;
*/
// Produce a word that mask the field "elem" in any polygon.
#define POLY_MASK(elem) (1 << (unsigned char)(&poly_dummy->elem - (double *)poly_dummy))

#define POLY_CLIP_OUT 0		/* polygon entirely outside box */
#define POLY_CLIP_PARTIAL 1	/* polygon partially inside */
#define POLY_CLIP_IN 2		/* polygon entirely inside box */

extern Poly_vert *poly_dummy;	/* used superficially by POLY_MASK macro */

void	poly_print(/* str, p */);
void	poly_vert_label(/* str, mask */);
void	poly_vert_print(/* str, v, mask */);
int	poly_clip_to_box(/* p1, box */);
void	poly_clip_to_halfspace(/* p, q, index, sign, k, name */);
//void	poly_scan(Poly * p, TRect *win, void (*pixelproc)(int x, int y, Poly *) );

/*
 * poly_scan: Scan convert a polygon, calling pixelproc at each pixel with an
 * interpolated Poly_vert structure.  Polygon can be clockwise or ccw.
 * Polygon is clipped in 2-D to win, the screen space window.
 *
 * Scan conversion is done on the basis of Poly_vert fields sx and sy.
 * These two must always be interpolated, and only they have special meaning
 * to this code; any other fields are blindly interpolated regardless of
 * their semantics.
 *
 * The pixelproc subroutine takes the arguments:
 *
 *	pixelproc(x, y, point)
 *	int x, y;
 *	Poly_vert *point;
 *
 * All the fields of point indicated by p->mask will be valid inside pixelproc
 * except sx and sy.  If they were computed, they would have values
 * sx=x+.5 and sy=y+.5, since sampling is done at pixel centers.
 */

void poly_scan(register Poly *p, TRect *win,
    void (*pixelproc)(int, int, Poly_vert*));


Poly_vert polPolyDummy, *poly_dummy=&polPolyDummy;		/* used superficially by POLY_MASK macro */
unsigned long lScannedSurface;

void (*fFriendlyCrosserPixelProc)(TPoint pt, void *pArgs)=NULL;
void *pFriendlyCrossedArguments=NULL;

void fFriendlyCallerPixelProc(int x, int y, Poly_vert *)
  {
  lScannedSurface++;
  fFriendlyCrosserPixelProc(TPoint(x,y),pFriendlyCrossedArguments);
  }
// returns the surface of the scanned polygon.
unsigned long ScanPolygon(TPoint *apPolygon, unsigned int wNbPoints, TRect rScanningWindow,
    void (*fPixelProc)(TPoint pt, void *pArgs), void *pArgs)
  {
  if (wNbPoints>POLY_NMAX)
    return 0;

  Poly GemPoly;
  GemPoly.n=wNbPoints;
  GemPoly.mask= POLY_MASK(sx) | POLY_MASK(sy); // Macro

  double *pdMemCleaner=(double *)GemPoly.vert;
  for (unsigned int w=0; w<wNbPoints*16; w++)
    *pdMemCleaner++=0;

  for (w=0; w<wNbPoints; w++)
    {GemPoly.vert[w].sx=apPolygon[w].x;GemPoly.vert[w].sy=apPolygon[w].y;}

  if (pFriendlyCrossedArguments || fFriendlyCrosserPixelProc || lScannedSurface)
    return 0;

  pFriendlyCrossedArguments=pArgs;
  fFriendlyCrosserPixelProc=fPixelProc;
  lScannedSurface=0;

  poly_scan(&GemPoly, &rScanningWindow, fFriendlyCallerPixelProc);
  unsigned long l=lScannedSurface;

  pFriendlyCrossedArguments=fFriendlyCrosserPixelProc=NULL;
  lScannedSurface=0;


  return l;
  }


/*
 * incrementalize_y: put intersection of line Y=y+.5 with edge between points
 * p1 and p2 in p, put change with respect to y in dp
 */
/*
static void scanline(int y,Poly_vert * l,Poly_vert * r,TRect * win,
    void (*pixelproc)(), int mask);
*/
void scanline(int y,Poly_vert * l,Poly_vert * r,TRect *win,
    void (*pixelproc)(int, int, Poly_vert*), int mask);

void incrementalize_y(register double *p1, register double *p2,
    register double *p, register double *dp, int y, register int mask);
void incrementalize_x(register double *p1, register double *p2,
    register double *p, register double *dp, int x, register int mask);

void increment(double *p, double *dp, int mask);


static void incrementalize_y(register double *p1, register double *p2,
    register double *p, register double *dp,
    int y, register int mask)
{
    double dy, frac;

    dy = ((Poly_vert *)p2)->sy - ((Poly_vert *)p1)->sy;
    if (dy==0.) dy = 1.;
    frac = y+.5 - ((Poly_vert *)p1)->sy;

    for (; mask!=0; mask>>=1, p1++, p2++, p++, dp++)
	if (mask&1) {
	    *dp = (*p2-*p1)/dy;
	    *p = *p1+*dp*frac;
	}
}

/*
 * incrementalize_x: put intersection of line X=x+.5 with edge between points
 * p1 and p2 in p, put change with respect to x in dp
 */

static void incrementalize_x(register double *p1, register double *p2, register double *p, register double *dp,
    int x, register int mask)
{
    double dx, frac;

    dx = ((Poly_vert *)p2)->sx - ((Poly_vert *)p1)->sx;
    if (dx==0.) dx = 1.;
    frac = x+.5 - ((Poly_vert *)p1)->sx;

    for (; mask!=0; mask>>=1, p1++, p2++, p++, dp++)
	if (mask&1) {
	    *dp = (*p2-*p1)/dx;
	    *p = *p1+*dp*frac;
	}
}


void poly_scan(register Poly *p, TRect *win, void (*pixelproc)(int, int, Poly_vert*))
/*
register Poly *p;	      //	/* polygon
TRect *win;		      //	/* 2-D screen space clipping window
void (*pixelproc)();	      //	/* procedure called at each pixel
*/ 
{
    register int i, li, ri, y, ly, ry, top, rem, mask;
    double ymin;
    Poly_vert l, r, dl, dr;

    if (p->n>POLY_NMAX) {
	fprintf(stderr, "poly_scan: too many vertices: %d\n", p->n);
	return;
    }

    ymin = HUGE_VAL;
    for (i=0; i<p->n; i++)		/* find top vertex (y points down) */
	if (p->vert[i].sy < ymin) {
	    ymin = p->vert[i].sy;
	    top = i;
	}

    li = ri = top;			/* left and right vertex indices */
    rem = p->n;				/* number of vertices remaining */
    y = ceil(ymin-.5);			/* current scan line */
    ly = ry = y-1;			/* lower end of left & right edges */
    mask = p->mask & ~POLY_MASK(sy);	/* stop interpolating screen y */

    while (rem>0) {	/* scan in y, activating new edges on left & right */
			/* as scan line passes over new vertices */

	while (ly<=y && rem>0) {	/* advance left edge? */
	    rem--;
	    i = li-1;			/* step ccw down left side */
	    if (i<0) i = p->n-1;
	    incrementalize_y((double *)&p->vert[li], (double *)&p->vert[i],
                (double *)&l, (double *)&dl, y, mask);
	    ly = floor(p->vert[i].sy+.5);
	    li = i;
	}
	while (ry<=y && rem>0) {	/* advance right edge? */
	    rem--;
	    i = ri+1;			/* step cw down right edge */
	    if (i>=p->n) i = 0;
	    incrementalize_y((double *)&p->vert[ri], (double *)&p->vert[i], (double *)&r, (double *)&dr, y, mask);
	    ry = floor(p->vert[i].sy+.5);
	    ri = i;
	}

	while (y<ly && y<ry) {	    /* do scanlines till end of l or r edge */
	    if (y>=win->top && y<=win->bottom)
		if (l.sx<=r.sx) scanline(y, &l, &r, win, pixelproc, mask);
		else		scanline(y, &r, &l, win, pixelproc, mask);
	    y++;
	    increment((double *)&l, (double *)&dl, mask);
	    increment((double *)&r, (double *)&dr, mask);
	}
    }
}

/* scanline: output scanline by sampling polygon at Y=y+.5 */


static void scanline(int y,Poly_vert * l,Poly_vert * r,TRect *win,
    void (*pixelproc)(int, int, Poly_vert*), int mask)
/*int y, mask;
Poly_vert *l, *r;
TRect *win;
void (*pixelproc)();
*/
{
    int x, lx, rx;
    Poly_vert p, dp;

    mask &= ~POLY_MASK(sx);		/* stop interpolating screen x */
    lx = ceil(l->sx-.5);
    if (lx<win->left) lx = win->left;
    rx = floor(r->sx-.5);
    if (rx>win->right) rx = win->right;
    if (lx>rx) return;
    incrementalize_x((double *)l, (double *)r, (double *)&p, (double *)&dp, lx, mask);
    for (x=lx; x<=rx; x++) {		/* scan in x, generating pixels */
	pixelproc(x, y, &p);
	increment((double *)&p, (double *)&dp, mask);
    }
}


static void increment(register double *p, register double *dp,
    register int mask)
{
    for (; mask!=0; mask>>=1, p++, dp++)
	if (mask&1)
	    *p += *dp;
}
